var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
function tr(text) { return text; }
loader.register_task(loader.Stage.JAVASCRIPT_INITIALIZING, {
    name: "certificate accept tester",
    function: () => __awaiter(this, void 0, void 0, function* () {
        const certificate_accept = settings.static_global(Settings.KEY_CERTIFICATE_CALLBACK, undefined);
        const container_success = $("#container-success").hide();
        if (!certificate_accept) {
            loader.critical_error(tr("Missing certificate callback data"), tr("Please reconnect manually."));
            throw "missing data";
        }
        log.info(LogCategory.IPC, tr("Using this instance as certificate callback. ID: %s"), certificate_accept);
        try {
            yield bipc.get_handler().post_certificate_accpected(certificate_accept);
            log.info(LogCategory.IPC, tr("Other instance has acknowledged out work. Closing this window."));
            let seconds = 5;
            let interval_id;
            interval_id = setInterval(() => {
                seconds--;
                $("#time-left").text(seconds.toString());
                if (seconds <= 0) {
                    clearTimeout(interval_id);
                    log.info(LogCategory.GENERAL, tr("Closing window"));
                    window.close();
                    return;
                }
            }, 1000);
            container_success.show();
        }
        catch (error) {
            log.warn(LogCategory.IPC, tr("Failed to successfully post certificate accept status: %o"), error);
            loader.critical_error(tr("Failed to emit success!"), tr("Please reconnect manually."));
        }
    }),
    priority: 10
});
var bipc;
(function (bipc) {
    function uuidv4() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            const r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    }
    class BasicIPCHandler {
        constructor() {
            this._channels = [];
            this._query_results = {};
            this._cert_accept_callbacks = {};
            this._cert_accept_succeeded = {};
        }
        setup() {
            this.unique_id = uuidv4(); /* lets get an unique identifier */
        }
        get_local_address() { return this.unique_id; }
        handle_message(message) {
            //log.trace(LogCategory.IPC, tr("Received message %o"), message);
            if (message.receiver === BasicIPCHandler.BROADCAST_UNIQUE_ID) {
                if (message.type == "process-query") {
                    log.debug(LogCategory.IPC, tr("Received a device query from %s."), message.sender);
                    this.send_message("process-query-response", {
                        request_query_id: message.data.query_id,
                        request_timestamp: message.data.timestamp,
                        device_id: this.unique_id,
                        protocol: BasicIPCHandler.PROTOCOL_VERSION
                    }, message.sender);
                    return;
                }
            }
            else if (message.receiver === this.unique_id) {
                if (message.type == "process-query-response") {
                    const response = message.data;
                    if (this._query_results[response.request_query_id])
                        this._query_results[response.request_query_id].push(response);
                    else {
                        log.warn(LogCategory.IPC, tr("Received a query response for an unknown request."));
                    }
                    return;
                }
                else if (message.type == "certificate-accept-callback") {
                    const data = message.data;
                    if (!this._cert_accept_callbacks[data.request_id]) {
                        log.warn(LogCategory.IPC, tr("Received certificate accept callback for an unknown request ID."));
                        return;
                    }
                    this._cert_accept_callbacks[data.request_id]();
                    delete this._cert_accept_callbacks[data.request_id];
                    this.send_message("certificate-accept-succeeded", {}, message.sender);
                    return;
                }
                else if (message.type == "certificate-accept-succeeded") {
                    if (!this._cert_accept_succeeded[message.sender]) {
                        log.warn(LogCategory.IPC, tr("Received certificate accept succeeded, but haven't a callback."));
                        return;
                    }
                    this._cert_accept_succeeded[message.sender]();
                    return;
                }
            }
            if (message.type === "channel") {
                const data = message.data;
                let channel_invoked = false;
                for (const channel of this._channels)
                    if (channel.channel_id === data.channel_id && (typeof (channel.target_id) === "undefined" || channel.target_id === message.sender)) {
                        if (channel.message_handler)
                            channel.message_handler(message.sender, message.receiver === BasicIPCHandler.BROADCAST_UNIQUE_ID, data);
                        channel_invoked = true;
                    }
                if (!channel_invoked) {
                    console.warn(tr("Received channel message for unknown channel (%s)"), data.channel_id);
                }
            }
        }
        create_channel(target_id, channel_id) {
            let channel = {
                target_id: target_id,
                channel_id: channel_id || uuidv4(),
                message_handler: undefined,
                send_message: (type, data, target) => {
                    if (typeof target !== "undefined") {
                        if (typeof channel.target_id === "string" && target != channel.target_id)
                            throw "target id does not match channel target";
                    }
                    this.send_message("channel", {
                        type: type,
                        data: data,
                        channel_id: channel.channel_id
                    }, target || channel.target_id || BasicIPCHandler.BROADCAST_UNIQUE_ID);
                }
            };
            this._channels.push(channel);
            return channel;
        }
        channels() { return this._channels; }
        delete_channel(channel) {
            this._channels = this._channels.filter(e => e !== channel);
        }
        query_processes(timeout) {
            return __awaiter(this, void 0, void 0, function* () {
                const query_id = uuidv4();
                this._query_results[query_id] = [];
                this.send_message("process-query", {
                    query_id: query_id,
                    timestamp: Date.now()
                });
                yield new Promise(resolve => setTimeout(resolve, timeout || 250));
                const result = this._query_results[query_id];
                delete this._query_results[query_id];
                return result;
            });
        }
        register_certificate_accept_callback(callback) {
            const id = uuidv4();
            this._cert_accept_callbacks[id] = callback;
            return this.unique_id + ":" + id;
        }
        post_certificate_accpected(id, timeout) {
            return new Promise((resolve, reject) => {
                const data = id.split(":");
                const timeout_id = setTimeout(() => {
                    delete this._cert_accept_succeeded[data[0]];
                    clearTimeout(timeout_id);
                    reject("timeout");
                }, timeout || 250);
                this._cert_accept_succeeded[data[0]] = () => {
                    delete this._cert_accept_succeeded[data[0]];
                    clearTimeout(timeout_id);
                    resolve();
                };
                this.send_message("certificate-accept-callback", {
                    request_id: data[1]
                }, data[0]);
            });
        }
    }
    BasicIPCHandler.BROADCAST_UNIQUE_ID = "00000000-0000-4000-0000-000000000000";
    BasicIPCHandler.PROTOCOL_VERSION = 1;
    bipc.BasicIPCHandler = BasicIPCHandler;
    class BroadcastChannelIPC extends BasicIPCHandler {
        constructor() {
            super();
        }
        setup() {
            super.setup();
            this.channel = new BroadcastChannel(BroadcastChannelIPC.CHANNEL_NAME);
            this.channel.onmessage = this.on_message.bind(this);
            this.channel.onmessageerror = this.on_error.bind(this);
        }
        on_message(event) {
            if (typeof (event.data) !== "string") {
                log.warn(LogCategory.IPC, tr("Received message with an invalid type (%s): %o"), typeof (event.data), event.data);
                return;
            }
            let message;
            try {
                message = JSON.parse(event.data);
            }
            catch (error) {
                log.error(LogCategory.IPC, tr("Received an invalid encoded message: %o"), event.data);
                return;
            }
            super.handle_message(message);
        }
        on_error(event) {
            log.warn(LogCategory.IPC, tr("Received error: %o"), event);
        }
        send_message(type, data, target) {
            const message = {};
            message.sender = this.unique_id;
            message.receiver = target ? target : BasicIPCHandler.BROADCAST_UNIQUE_ID;
            message.timestamp = Date.now();
            message.type = type;
            message.data = data;
            this.channel.postMessage(JSON.stringify(message));
        }
    }
    BroadcastChannelIPC.CHANNEL_NAME = "TeaSpeak-Web";
    let connect;
    (function (connect) {
        /* The connect process:
         *  1. Broadcast an offer
         *  2. Wait 50ms for all offer responses or until the first one respond with "ok"
         *  3. Select (if possible) on accepted offer and execute the connect
         */
        class ConnectHandler {
            constructor(ipc_handler) {
                this.callback_available = () => false;
                this.callback_execute = () => false;
                this._pending_connect_offers = [];
                this._pending_connects_requests = [];
                this.ipc_handler = ipc_handler;
            }
            setup() {
                this.ipc_channel = this.ipc_handler.create_channel(undefined, ConnectHandler.CHANNEL_NAME);
                this.ipc_channel.message_handler = this.on_message.bind(this);
            }
            on_message(sender, broadcast, message) {
                if (broadcast) {
                    if (message.type == "offer") {
                        const data = message.data;
                        const response = {
                            accepted: this.callback_available(data.data),
                            request_id: data.request_id
                        };
                        if (response.accepted) {
                            log.debug(LogCategory.IPC, tr("Received new connect offer from %s: %s"), sender, data.request_id);
                            const ld = {
                                remote_handler: sender,
                                data: data.data,
                                id: data.request_id,
                                timeout: 0
                            };
                            this._pending_connect_offers.push(ld);
                            ld.timeout = setTimeout(() => {
                                log.debug(LogCategory.IPC, tr("Dropping connect request %s, because we never received an execute."), ld.id);
                                this._pending_connect_offers.remove(ld);
                            }, 120 * 1000);
                        }
                        this.ipc_channel.send_message("offer-answer", response, sender);
                    }
                }
                else {
                    if (message.type == "offer-answer") {
                        const data = message.data;
                        const request = this._pending_connects_requests.find(e => e.id === data.request_id);
                        if (!request) {
                            log.warn(LogCategory.IPC, tr("Received connect offer answer with unknown request id (%s)."), data.request_id);
                            return;
                        }
                        if (!data.accepted) {
                            log.debug(LogCategory.IPC, tr("Client %s rejected the connect offer (%s)."), sender, request.id);
                            return;
                        }
                        if (request.remote_handler) {
                            log.debug(LogCategory.IPC, tr("Client %s accepted the connect offer (%s), but offer has already been accepted."), sender, request.id);
                            return;
                        }
                        log.debug(LogCategory.IPC, tr("Client %s accepted the connect offer (%s). Request local acceptance."), sender, request.id);
                        request.remote_handler = sender;
                        clearTimeout(request.timeout);
                        request.callback_avail().then(flag => {
                            if (!flag) {
                                request.callback_failed("local avail rejected");
                                return;
                            }
                            log.debug(LogCategory.IPC, tr("Executing connect with client %s"), request.remote_handler);
                            this.ipc_channel.send_message("execute", {
                                request_id: request.id
                            }, request.remote_handler);
                            request.timeout = setTimeout(() => {
                                request.callback_failed("connect execute timeout");
                            }, 1000);
                        }).catch(error => {
                            log.error(LogCategory.IPC, tr("Local avail callback caused an error: %o"), error);
                            request.callback_failed(tr("local avail callback caused an error"));
                        });
                    }
                    else if (message.type == "executed") {
                        const data = message.data;
                        const request = this._pending_connects_requests.find(e => e.id === data.request_id);
                        if (!request) {
                            log.warn(LogCategory.IPC, tr("Received connect executed with unknown request id (%s)."), data.request_id);
                            return;
                        }
                        if (request.remote_handler != sender) {
                            log.warn(LogCategory.IPC, tr("Received connect executed for request %s, but from wrong client: %s (expected %s)"), data.request_id, sender, request.remote_handler);
                            return;
                        }
                        log.debug(LogCategory.IPC, tr("Received connect executed response from client %s for request %s. Succeeded: %o (%s)"), sender, data.request_id, data.succeeded, data.message);
                        clearTimeout(request.timeout);
                        if (data.succeeded)
                            request.callback_success();
                        else
                            request.callback_failed(data.message);
                    }
                    else if (message.type == "execute") {
                        const data = message.data;
                        const request = this._pending_connect_offers.find(e => e.id === data.request_id);
                        if (!request) {
                            log.warn(LogCategory.IPC, tr("Received connect execute with unknown request id (%s)."), data.request_id);
                            return;
                        }
                        if (request.remote_handler != sender) {
                            log.warn(LogCategory.IPC, tr("Received connect execute for request %s, but from wrong client: %s (expected %s)"), data.request_id, sender, request.remote_handler);
                            return;
                        }
                        clearTimeout(request.timeout);
                        this._pending_connect_offers.remove(request);
                        log.debug(LogCategory.IPC, tr("Executing connect for %s"), data.request_id);
                        const cr = this.callback_execute(request.data);
                        const response = {
                            request_id: data.request_id,
                            succeeded: typeof (cr) !== "string" && cr,
                            message: typeof (cr) === "string" ? cr : "",
                        };
                        this.ipc_channel.send_message("executed", response, request.remote_handler);
                    }
                }
            }
            post_connect_request(data, callback_avail) {
                return new Promise((resolve, reject) => {
                    const pd = {
                        data: data,
                        id: uuidv4(),
                        timeout: 0,
                        callback_success: () => {
                            this._pending_connects_requests.remove(pd);
                            clearTimeout(pd.timeout);
                            resolve();
                        },
                        callback_failed: error => {
                            this._pending_connects_requests.remove(pd);
                            clearTimeout(pd.timeout);
                            reject(error);
                        },
                        callback_avail: callback_avail,
                    };
                    this._pending_connects_requests.push(pd);
                    this.ipc_channel.send_message("offer", {
                        request_id: pd.id,
                        data: pd.data
                    });
                    pd.timeout = setTimeout(() => {
                        pd.callback_failed("received no response to offer");
                    }, 50);
                });
            }
        }
        ConnectHandler.CHANNEL_NAME = "connect";
        connect.ConnectHandler = ConnectHandler;
    })(connect = bipc.connect || (bipc.connect = {}));
    let mproxy;
    (function (mproxy) {
        class MethodProxy {
            constructor(ipc_handler, connect_params) {
                this._proxied_methods = {};
                this._proxied_callbacks = {};
                this.ipc_handler = ipc_handler;
                this._ipc_parameters = connect_params;
                this._connected = false;
                this._slave = typeof (connect_params) !== "undefined";
                this._local = typeof (connect_params) !== "undefined" && connect_params.channel_id === "local" && connect_params.client_id === "local";
            }
            setup() {
                if (this._local) {
                    this._connected = true;
                    this.on_connected();
                }
                else {
                    if (this._slave)
                        this._ipc_channel = this.ipc_handler.create_channel(this._ipc_parameters.client_id, this._ipc_parameters.channel_id);
                    else
                        this._ipc_channel = this.ipc_handler.create_channel();
                    this._ipc_channel.message_handler = this._handle_message.bind(this);
                    if (this._slave)
                        this._ipc_channel.send_message("initialize", {});
                }
            }
            finalize() {
                if (!this._local) {
                    if (this._connected)
                        this._ipc_channel.send_message("finalize", {});
                    this.ipc_handler.delete_channel(this._ipc_channel);
                    this._ipc_channel = undefined;
                }
                for (const promise of Object.values(this._proxied_callbacks))
                    promise.reject("disconnected");
                this._proxied_callbacks = {};
                this._connected = false;
                this.on_disconnected();
            }
            register_method(method) {
                let method_name;
                if (typeof method === "function") {
                    log.debug(LogCategory.IPC, tr("Registering method proxy for %s"), method.name);
                    method_name = method.name;
                }
                else {
                    log.debug(LogCategory.IPC, tr("Registering method proxy for %s"), method);
                    method_name = method;
                }
                if (!this[method_name])
                    throw "method is missing in current object";
                this._proxied_methods[method_name] = this[method_name];
                if (!this._local) {
                    this[method_name] = (...args) => {
                        if (!this._connected)
                            return Promise.reject("not connected");
                        const proxy_callback = {
                            promise_id: uuidv4()
                        };
                        this._proxied_callbacks[proxy_callback.promise_id] = proxy_callback;
                        proxy_callback.promise = new Promise((resolve, reject) => {
                            proxy_callback.resolve = resolve;
                            proxy_callback.reject = reject;
                        });
                        this._ipc_channel.send_message("invoke", {
                            promise_id: proxy_callback.promise_id,
                            arguments: [...args],
                            method_name: method_name
                        });
                        return proxy_callback.promise;
                    };
                }
            }
            _handle_message(remote_id, boradcast, message) {
                if (message.type === "finalize") {
                    this._handle_finalize();
                }
                else if (message.type === "initialize") {
                    this._handle_remote_callback(remote_id);
                }
                else if (message.type === "invoke") {
                    this._handle_invoke(message.data);
                }
                else if (message.type === "result") {
                    this._handle_result(message.data);
                }
            }
            _handle_finalize() {
                this.on_disconnected();
                this.finalize();
                this._connected = false;
            }
            _handle_remote_callback(remote_id) {
                if (!this._ipc_channel.target_id) {
                    if (this._slave)
                        throw "initialize wrong state!";
                    this._ipc_channel.target_id = remote_id; /* now we're able to send messages */
                    this.on_connected();
                    this._ipc_channel.send_message("initialize", true);
                }
                else {
                    if (!this._slave)
                        throw "initialize wrong state!";
                    this.on_connected();
                }
                this._connected = true;
            }
            _send_result(promise_id, success, message) {
                this._ipc_channel.send_message("result", {
                    promise_id: promise_id,
                    result: message,
                    success: success
                });
            }
            _handle_invoke(data) {
                if (this._proxied_methods[data.method_name])
                    throw "we could not invoke a local proxied method!";
                if (!this[data.method_name]) {
                    this._send_result(data.promise_id, false, "missing method");
                    return;
                }
                try {
                    log.info(LogCategory.IPC, tr("Invoking method %s with arguments: %o"), data.method_name, data.arguments);
                    const promise = this[data.method_name](...data.arguments);
                    promise.then(result => {
                        log.info(LogCategory.IPC, tr("Result: %o"), result);
                        this._send_result(data.promise_id, true, result);
                    }).catch(error => {
                        this._send_result(data.promise_id, false, error);
                    });
                }
                catch (error) {
                    this._send_result(data.promise_id, false, error);
                    return;
                }
            }
            _handle_result(data) {
                if (!this._proxied_callbacks[data.promise_id]) {
                    console.warn(tr("Received proxy method result for unknown promise"));
                    return;
                }
                const callback = this._proxied_callbacks[data.promise_id];
                delete this._proxied_callbacks[data.promise_id];
                if (data.success)
                    callback.resolve(data.result);
                else
                    callback.reject(data.result);
            }
            generate_connect_parameters() {
                if (this._slave)
                    throw "only masters can generate connect parameters!";
                if (!this._ipc_channel)
                    throw "please call setup() before";
                return {
                    channel_id: this._ipc_channel.channel_id,
                    client_id: this.ipc_handler.get_local_address()
                };
            }
            is_slave() { return this._local || this._slave; } /* the popout modal */
            is_master() { return this._local || !this._slave; } /* the host (teaweb application) */
        }
        mproxy.MethodProxy = MethodProxy;
    })(mproxy = bipc.mproxy || (bipc.mproxy = {}));
    let handler;
    let connect_handler;
    function setup() {
        if (!supported())
            return;
        handler = new BroadcastChannelIPC();
        handler.setup();
        connect_handler = new connect.ConnectHandler(handler);
        connect_handler.setup();
    }
    bipc.setup = setup;
    function get_handler() {
        return handler;
    }
    bipc.get_handler = get_handler;
    function get_connect_handler() {
        return connect_handler;
    }
    bipc.get_connect_handler = get_connect_handler;
    function supported() {
        /* ios does not support this */
        return typeof (window.BroadcastChannel) !== "undefined";
    }
    bipc.supported = supported;
})(bipc || (bipc = {}));
var KeyCode;
(function (KeyCode) {
    KeyCode[KeyCode["KEY_CANCEL"] = 3] = "KEY_CANCEL";
    KeyCode[KeyCode["KEY_HELP"] = 6] = "KEY_HELP";
    KeyCode[KeyCode["KEY_BACK_SPACE"] = 8] = "KEY_BACK_SPACE";
    KeyCode[KeyCode["KEY_TAB"] = 9] = "KEY_TAB";
    KeyCode[KeyCode["KEY_CLEAR"] = 12] = "KEY_CLEAR";
    KeyCode[KeyCode["KEY_RETURN"] = 13] = "KEY_RETURN";
    KeyCode[KeyCode["KEY_ENTER"] = 14] = "KEY_ENTER";
    KeyCode[KeyCode["KEY_SHIFT"] = 16] = "KEY_SHIFT";
    KeyCode[KeyCode["KEY_CONTROL"] = 17] = "KEY_CONTROL";
    KeyCode[KeyCode["KEY_ALT"] = 18] = "KEY_ALT";
    KeyCode[KeyCode["KEY_PAUSE"] = 19] = "KEY_PAUSE";
    KeyCode[KeyCode["KEY_CAPS_LOCK"] = 20] = "KEY_CAPS_LOCK";
    KeyCode[KeyCode["KEY_ESCAPE"] = 27] = "KEY_ESCAPE";
    KeyCode[KeyCode["KEY_SPACE"] = 32] = "KEY_SPACE";
    KeyCode[KeyCode["KEY_PAGE_UP"] = 33] = "KEY_PAGE_UP";
    KeyCode[KeyCode["KEY_PAGE_DOWN"] = 34] = "KEY_PAGE_DOWN";
    KeyCode[KeyCode["KEY_END"] = 35] = "KEY_END";
    KeyCode[KeyCode["KEY_HOME"] = 36] = "KEY_HOME";
    KeyCode[KeyCode["KEY_LEFT"] = 37] = "KEY_LEFT";
    KeyCode[KeyCode["KEY_UP"] = 38] = "KEY_UP";
    KeyCode[KeyCode["KEY_RIGHT"] = 39] = "KEY_RIGHT";
    KeyCode[KeyCode["KEY_DOWN"] = 40] = "KEY_DOWN";
    KeyCode[KeyCode["KEY_PRINTSCREEN"] = 44] = "KEY_PRINTSCREEN";
    KeyCode[KeyCode["KEY_INSERT"] = 45] = "KEY_INSERT";
    KeyCode[KeyCode["KEY_DELETE"] = 46] = "KEY_DELETE";
    KeyCode[KeyCode["KEY_0"] = 48] = "KEY_0";
    KeyCode[KeyCode["KEY_1"] = 49] = "KEY_1";
    KeyCode[KeyCode["KEY_2"] = 50] = "KEY_2";
    KeyCode[KeyCode["KEY_3"] = 51] = "KEY_3";
    KeyCode[KeyCode["KEY_4"] = 52] = "KEY_4";
    KeyCode[KeyCode["KEY_5"] = 53] = "KEY_5";
    KeyCode[KeyCode["KEY_6"] = 54] = "KEY_6";
    KeyCode[KeyCode["KEY_7"] = 55] = "KEY_7";
    KeyCode[KeyCode["KEY_8"] = 56] = "KEY_8";
    KeyCode[KeyCode["KEY_9"] = 57] = "KEY_9";
    KeyCode[KeyCode["KEY_SEMICOLON"] = 59] = "KEY_SEMICOLON";
    KeyCode[KeyCode["KEY_EQUALS"] = 61] = "KEY_EQUALS";
    KeyCode[KeyCode["KEY_A"] = 65] = "KEY_A";
    KeyCode[KeyCode["KEY_B"] = 66] = "KEY_B";
    KeyCode[KeyCode["KEY_C"] = 67] = "KEY_C";
    KeyCode[KeyCode["KEY_D"] = 68] = "KEY_D";
    KeyCode[KeyCode["KEY_E"] = 69] = "KEY_E";
    KeyCode[KeyCode["KEY_F"] = 70] = "KEY_F";
    KeyCode[KeyCode["KEY_G"] = 71] = "KEY_G";
    KeyCode[KeyCode["KEY_H"] = 72] = "KEY_H";
    KeyCode[KeyCode["KEY_I"] = 73] = "KEY_I";
    KeyCode[KeyCode["KEY_J"] = 74] = "KEY_J";
    KeyCode[KeyCode["KEY_K"] = 75] = "KEY_K";
    KeyCode[KeyCode["KEY_L"] = 76] = "KEY_L";
    KeyCode[KeyCode["KEY_M"] = 77] = "KEY_M";
    KeyCode[KeyCode["KEY_N"] = 78] = "KEY_N";
    KeyCode[KeyCode["KEY_O"] = 79] = "KEY_O";
    KeyCode[KeyCode["KEY_P"] = 80] = "KEY_P";
    KeyCode[KeyCode["KEY_Q"] = 81] = "KEY_Q";
    KeyCode[KeyCode["KEY_R"] = 82] = "KEY_R";
    KeyCode[KeyCode["KEY_S"] = 83] = "KEY_S";
    KeyCode[KeyCode["KEY_T"] = 84] = "KEY_T";
    KeyCode[KeyCode["KEY_U"] = 85] = "KEY_U";
    KeyCode[KeyCode["KEY_V"] = 86] = "KEY_V";
    KeyCode[KeyCode["KEY_W"] = 87] = "KEY_W";
    KeyCode[KeyCode["KEY_X"] = 88] = "KEY_X";
    KeyCode[KeyCode["KEY_Y"] = 89] = "KEY_Y";
    KeyCode[KeyCode["KEY_Z"] = 90] = "KEY_Z";
    KeyCode[KeyCode["KEY_LEFT_CMD"] = 91] = "KEY_LEFT_CMD";
    KeyCode[KeyCode["KEY_RIGHT_CMD"] = 93] = "KEY_RIGHT_CMD";
    KeyCode[KeyCode["KEY_CONTEXT_MENU"] = 93] = "KEY_CONTEXT_MENU";
    KeyCode[KeyCode["KEY_NUMPAD0"] = 96] = "KEY_NUMPAD0";
    KeyCode[KeyCode["KEY_NUMPAD1"] = 97] = "KEY_NUMPAD1";
    KeyCode[KeyCode["KEY_NUMPAD2"] = 98] = "KEY_NUMPAD2";
    KeyCode[KeyCode["KEY_NUMPAD3"] = 99] = "KEY_NUMPAD3";
    KeyCode[KeyCode["KEY_NUMPAD4"] = 100] = "KEY_NUMPAD4";
    KeyCode[KeyCode["KEY_NUMPAD5"] = 101] = "KEY_NUMPAD5";
    KeyCode[KeyCode["KEY_NUMPAD6"] = 102] = "KEY_NUMPAD6";
    KeyCode[KeyCode["KEY_NUMPAD7"] = 103] = "KEY_NUMPAD7";
    KeyCode[KeyCode["KEY_NUMPAD8"] = 104] = "KEY_NUMPAD8";
    KeyCode[KeyCode["KEY_NUMPAD9"] = 105] = "KEY_NUMPAD9";
    KeyCode[KeyCode["KEY_MULTIPLY"] = 106] = "KEY_MULTIPLY";
    KeyCode[KeyCode["KEY_ADD"] = 107] = "KEY_ADD";
    KeyCode[KeyCode["KEY_SEPARATOR"] = 108] = "KEY_SEPARATOR";
    KeyCode[KeyCode["KEY_SUBTRACT"] = 109] = "KEY_SUBTRACT";
    KeyCode[KeyCode["KEY_DECIMAL"] = 110] = "KEY_DECIMAL";
    KeyCode[KeyCode["KEY_DIVIDE"] = 111] = "KEY_DIVIDE";
    KeyCode[KeyCode["KEY_F1"] = 112] = "KEY_F1";
    KeyCode[KeyCode["KEY_F2"] = 113] = "KEY_F2";
    KeyCode[KeyCode["KEY_F3"] = 114] = "KEY_F3";
    KeyCode[KeyCode["KEY_F4"] = 115] = "KEY_F4";
    KeyCode[KeyCode["KEY_F5"] = 116] = "KEY_F5";
    KeyCode[KeyCode["KEY_F6"] = 117] = "KEY_F6";
    KeyCode[KeyCode["KEY_F7"] = 118] = "KEY_F7";
    KeyCode[KeyCode["KEY_F8"] = 119] = "KEY_F8";
    KeyCode[KeyCode["KEY_F9"] = 120] = "KEY_F9";
    KeyCode[KeyCode["KEY_F10"] = 121] = "KEY_F10";
    KeyCode[KeyCode["KEY_F11"] = 122] = "KEY_F11";
    KeyCode[KeyCode["KEY_F12"] = 123] = "KEY_F12";
    KeyCode[KeyCode["KEY_F13"] = 124] = "KEY_F13";
    KeyCode[KeyCode["KEY_F14"] = 125] = "KEY_F14";
    KeyCode[KeyCode["KEY_F15"] = 126] = "KEY_F15";
    KeyCode[KeyCode["KEY_F16"] = 127] = "KEY_F16";
    KeyCode[KeyCode["KEY_F17"] = 128] = "KEY_F17";
    KeyCode[KeyCode["KEY_F18"] = 129] = "KEY_F18";
    KeyCode[KeyCode["KEY_F19"] = 130] = "KEY_F19";
    KeyCode[KeyCode["KEY_F20"] = 131] = "KEY_F20";
    KeyCode[KeyCode["KEY_F21"] = 132] = "KEY_F21";
    KeyCode[KeyCode["KEY_F22"] = 133] = "KEY_F22";
    KeyCode[KeyCode["KEY_F23"] = 134] = "KEY_F23";
    KeyCode[KeyCode["KEY_F24"] = 135] = "KEY_F24";
    KeyCode[KeyCode["KEY_NUM_LOCK"] = 144] = "KEY_NUM_LOCK";
    KeyCode[KeyCode["KEY_SCROLL_LOCK"] = 145] = "KEY_SCROLL_LOCK";
    KeyCode[KeyCode["KEY_COMMA"] = 188] = "KEY_COMMA";
    KeyCode[KeyCode["KEY_PERIOD"] = 190] = "KEY_PERIOD";
    KeyCode[KeyCode["KEY_SLASH"] = 191] = "KEY_SLASH";
    KeyCode[KeyCode["KEY_BACK_QUOTE"] = 192] = "KEY_BACK_QUOTE";
    KeyCode[KeyCode["KEY_OPEN_BRACKET"] = 219] = "KEY_OPEN_BRACKET";
    KeyCode[KeyCode["KEY_BACK_SLASH"] = 220] = "KEY_BACK_SLASH";
    KeyCode[KeyCode["KEY_CLOSE_BRACKET"] = 221] = "KEY_CLOSE_BRACKET";
    KeyCode[KeyCode["KEY_QUOTE"] = 222] = "KEY_QUOTE";
    KeyCode[KeyCode["KEY_META"] = 224] = "KEY_META";
})(KeyCode || (KeyCode = {}));
var ppt;
(function (ppt) {
    let EventType;
    (function (EventType) {
        EventType[EventType["KEY_PRESS"] = 0] = "KEY_PRESS";
        EventType[EventType["KEY_RELEASE"] = 1] = "KEY_RELEASE";
        EventType[EventType["KEY_TYPED"] = 2] = "KEY_TYPED";
    })(EventType = ppt.EventType || (ppt.EventType = {}));
    let SpecialKey;
    (function (SpecialKey) {
        SpecialKey[SpecialKey["CTRL"] = 0] = "CTRL";
        SpecialKey[SpecialKey["WINDOWS"] = 1] = "WINDOWS";
        SpecialKey[SpecialKey["SHIFT"] = 2] = "SHIFT";
        SpecialKey[SpecialKey["ALT"] = 3] = "ALT";
    })(SpecialKey = ppt.SpecialKey || (ppt.SpecialKey = {}));
    function key_description(key) {
        let result = "";
        if (key.key_shift)
            result += " + " + tr("Shift");
        if (key.key_alt)
            result += " + " + tr("Alt");
        if (key.key_ctrl)
            result += " + " + tr("CTRL");
        if (key.key_windows)
            result += " + " + tr("Win");
        if (!result && !key.key_code)
            return tr("unset");
        if (key.key_code)
            result += " + " + key.key_code;
        return result.substr(3);
    }
    ppt.key_description = key_description;
})(ppt || (ppt = {}));
/// <reference path="../../PPTListener.ts" />
var ElementType;
(function (ElementType) {
    ElementType[ElementType["HEADER"] = 0] = "HEADER";
    ElementType[ElementType["BODY"] = 1] = "BODY";
    ElementType[ElementType["FOOTER"] = 2] = "FOOTER";
})(ElementType || (ElementType = {}));
const ModalFunctions = {
    divify: function (val) {
        if (val.length > 1)
            return $.spawn("div").append(val);
        return val;
    },
    jqueriefy: function (val, type) {
        if (typeof (val) === "function")
            val = val();
        if (val instanceof jQuery)
            return val;
        if (Array.isArray(val)) {
            if (val.length == 0)
                return undefined;
            return val.map(e => this.jqueriefy(e));
        }
        switch (typeof val) {
            case "string":
                if (type == ElementType.HEADER)
                    return $.spawn("div").addClass("modal-title").text(val);
                return $("<div>" + val + "</div>");
            case "object": return val;
            case "undefined":
                return undefined;
            default:
                console.error(("Invalid type %o"), typeof val);
                return $();
        }
    },
    warpProperties(data) {
        if (data instanceof ModalProperties)
            return data;
        else {
            const props = new ModalProperties();
            for (const key of Object.keys(data))
                props[key] = data[key];
            return props;
        }
    }
};
class ModalProperties {
    constructor() {
        this.header = () => "HEADER";
        this.body = () => "BODY";
        this.footer = () => "FOOTER";
        this.closeListener = () => { };
        this.height = "auto";
        this.closeable = true;
        this.template_properties = {};
        this.trigger_tab = true;
        this.full_size = false;
    }
    registerCloseListener(listener) {
        if (this.closeListener) {
            if ($.isArray(this.closeListener))
                this.closeListener.push(listener);
            else
                this.closeListener = [this.closeListener, listener];
        }
        else
            this.closeListener = listener;
        return this;
    }
    triggerClose() {
        if ($.isArray(this.closeListener))
            for (let listener of this.closeListener)
                listener();
        else
            this.closeListener();
    }
}
$(document).on('mousedown', (event) => {
    /* pageX or pageY are undefined if this is an event executed via .trigger('click'); */
    if (_global_modal_count == 0 || typeof (event.pageX) === "undefined" || typeof (event.pageY) === "undefined")
        return;
    let element = event.target;
    do {
        if (element.classList.contains('modal-content'))
            break;
        if (!element.classList.contains('modal'))
            continue;
        if (element == _global_modal_last && _global_modal_last_time + 100 > Date.now())
            break;
        $(element).find("> .modal-dialog > .modal-content > .modal-header .button-modal-close").trigger('click');
        break;
    } while ((element = element.parentElement));
});
let _global_modal_count = 0;
let _global_modal_last;
let _global_modal_last_time;
class Modal {
    constructor(props) {
        this.open_listener = [];
        this.close_listener = [];
        this.properties = props;
        this.shown = false;
    }
    get htmlTag() {
        if (!this._htmlTag)
            this._create();
        return this._htmlTag;
    }
    _create() {
        const header = ModalFunctions.jqueriefy(this.properties.header, ElementType.HEADER);
        const body = ModalFunctions.jqueriefy(this.properties.body, ElementType.BODY);
        const footer = ModalFunctions.jqueriefy(this.properties.footer, ElementType.FOOTER);
        //FIXME: cache template
        const template = $(this.properties.template || "#tmpl_modal");
        const properties = {
            modal_header: header,
            modal_body: body,
            modal_footer: footer,
            closeable: this.properties.closeable,
            full_size: this.properties.full_size
        };
        if (this.properties.template_properties)
            Object.assign(properties, this.properties.template_properties);
        const tag = template.renderTag(properties);
        if (typeof (this.properties.width) !== "undefined" && typeof (this.properties.min_width) !== "undefined")
            tag.find(".modal-content")
                .css("min-width", this.properties.min_width)
                .css("width", this.properties.width);
        else if (typeof (this.properties.width) !== "undefined") //Legacy support
            tag.find(".modal-content").css("min-width", this.properties.width);
        else if (typeof (this.properties.min_width) !== "undefined")
            tag.find(".modal-content").css("min-width", this.properties.min_width);
        this.close_elements = tag.find(".button-modal-close");
        this.close_elements.toggle(this.properties.closeable).on('click', event => {
            if (this.properties.closeable)
                this.close();
        });
        this._htmlTag = tag;
        this._htmlTag.find("input").on('change', event => {
            $(event.target).parents(".form-group").toggleClass('is-filled', !!event.target.value);
        });
        //TODO: After the animation!
        this._htmlTag.on('hide.bs.modal', event => !this.properties.closeable || this.close());
        this._htmlTag.on('hidden.bs.modal', event => this._htmlTag.remove());
    }
    open() {
        if (this.shown)
            return;
        _global_modal_last_time = Date.now();
        _global_modal_last = this.htmlTag[0];
        this.shown = true;
        this.htmlTag.appendTo($("body"));
        _global_modal_count++;
        this.htmlTag.show();
        setTimeout(() => this.htmlTag.addClass('shown'), 0);
        setTimeout(() => {
            for (const listener of this.open_listener)
                listener();
            this.htmlTag.find(".tab").trigger('tab.resize');
        }, 300);
    }
    close() {
        if (!this.shown)
            return;
        _global_modal_count--;
        this.shown = false;
        this.htmlTag.removeClass('shown');
        setTimeout(() => {
            this.htmlTag.remove();
            this._htmlTag = undefined;
        }, 300);
        this.properties.triggerClose();
        for (const listener of this.close_listener)
            listener();
    }
    set_closeable(flag) {
        if (flag === this.properties.closeable)
            return;
        this.properties.closeable = flag;
        this.close_elements.toggle(flag);
    }
}
function createModal(data) {
    return new Modal(ModalFunctions.warpProperties(data));
}
class InputModalProperties extends ModalProperties {
}
function createInputModal(headMessage, question, validator, callback, props = {}) {
    props = ModalFunctions.warpProperties(props);
    props.template_properties || (props.template_properties = {});
    props.template_properties.field_title = props.field_title;
    props.template_properties.field_label = props.field_label;
    props.template_properties.field_placeholder = props.field_placeholder;
    props.template_properties.error_message = props.error_message;
    props.template = "#tmpl_modal_input";
    props.header = headMessage;
    props.template_properties.question = ModalFunctions.jqueriefy(question);
    const modal = createModal(props);
    const input = modal.htmlTag.find(".container-value input");
    const button_cancel = modal.htmlTag.find(".button-cancel");
    const button_submit = modal.htmlTag.find(".button-submit");
    let submited = false;
    input.on('keyup change', event => {
        const str = input.val();
        const valid = str !== undefined && validator(str);
        input.attr("pattern", valid ? null : "^[a]{1000}$").toggleClass("is-invalid", !valid);
        button_submit.prop("disabled", !valid);
    });
    input.on('keydown', event => {
        if (event.keyCode !== KeyCode.KEY_RETURN || event.shiftKey)
            return;
        if (button_submit.prop("disabled"))
            return;
        button_submit.trigger('click');
    });
    button_submit.on('click', event => {
        if (!submited) {
            submited = true;
            const str = input.val();
            if (str !== undefined && validator(str))
                callback(str);
            else
                callback(false);
        }
        modal.close();
    }).prop("disabled", !validator("")); /* disabled if empty input isn't allowed */
    button_cancel.on('click', event => {
        if (!submited) {
            submited = true;
            callback(false);
        }
        modal.close();
    });
    modal.open_listener.push(() => input.focus());
    modal.close_listener.push(() => button_cancel.trigger('click'));
    return modal;
}
function createErrorModal(header, message, props = { footer: undefined }) {
    props = ModalFunctions.warpProperties(props);
    (props.template_properties || (props.template_properties = {})).header_class = "modal-header-error";
    props.header = header;
    props.body = message;
    const modal = createModal(props);
    modal.htmlTag.find(".modal-body").addClass("modal-error");
    return modal;
}
function createInfoModal(header, message, props = { footer: undefined }) {
    props = ModalFunctions.warpProperties(props);
    (props.template_properties || (props.template_properties = {})).header_class = "modal-header-info";
    props.header = header;
    props.body = message;
    const modal = createModal(props);
    modal.htmlTag.find(".modal-body").addClass("modal-info");
    return modal;
}
$.fn.modalize = function (entry_callback, properties) {
    properties = properties || {};
    entry_callback = entry_callback || ((a, b, c) => undefined);
    let tag_modal = this[0].tagName.toLowerCase() == "modal" ? this : undefined; /* TODO may throw exception? */
    let tag_head = tag_modal ? tag_modal.find("modal-header") : ModalFunctions.jqueriefy(properties.header);
    let tag_body = tag_modal ? tag_modal.find("modal-body") : this;
    let tag_footer = tag_modal ? tag_modal.find("modal-footer") : ModalFunctions.jqueriefy(properties.footer);
    const result = entry_callback(tag_head, tag_body, tag_footer) || {};
    properties.header = result.header || tag_head;
    properties.body = result.body || tag_body;
    properties.footer = result.footer || tag_footer;
    return createModal(properties);
};
/// <reference path="ui/elements/modal.ts" />
//Used by CertAccept popup
if (typeof (customElements) !== "undefined") {
    try {
        class X_Properties extends HTMLElement {
        }
        class X_Property extends HTMLElement {
        }
        customElements.define('x-properties', X_Properties, { extends: 'div' });
        customElements.define('x-property', X_Property, { extends: 'div' });
    }
    catch (error) {
        console.warn("failed to define costum elements");
    }
}
class SettingsBase {
    static transformStO(input, _default, default_type) {
        default_type = default_type || typeof _default;
        if (typeof input === "undefined")
            return _default;
        if (default_type === "string")
            return input;
        else if (default_type === "number")
            return parseInt(input);
        else if (default_type === "boolean")
            return (input == "1" || input == "true");
        else if (default_type === "undefined")
            return input;
        return JSON.parse(input);
    }
    static transformOtS(input) {
        if (typeof input === "string")
            return input;
        else if (typeof input === "number")
            return input.toString();
        else if (typeof input === "boolean")
            return input ? "1" : "0";
        else if (typeof input === "undefined")
            return undefined;
        return JSON.stringify(input);
    }
    static resolveKey(key, _default, resolver, default_type) {
        let value = resolver(key.key);
        if (!value) {
            /* trying fallbacks */
            for (const fallback of key.fallback_keys || []) {
                value = resolver(fallback);
                if (typeof (value) === "string") {
                    /* fallback key succeeded */
                    const importer = (key.fallback_imports || {})[fallback];
                    if (importer)
                        return importer(value);
                    break;
                }
            }
        }
        if (typeof (value) !== 'string')
            return _default;
        return SettingsBase.transformStO(value, _default, default_type);
    }
    static keyify(key) {
        if (typeof (key) === "string")
            return { key: key };
        if (typeof (key) === "object" && key.key)
            return key;
        throw "key is not a key";
    }
}
SettingsBase.UPDATE_DIRECT = true;
class StaticSettings extends SettingsBase {
    constructor(_reserved = undefined) {
        super();
        if (_reserved && !StaticSettings._instance) {
            this._staticPropsTag = $("#properties");
            this.initializeStatic();
        }
        else {
            this._handle = StaticSettings.instance;
        }
    }
    static get instance() {
        if (!this._instance)
            this._instance = new StaticSettings(true);
        return this._instance;
    }
    initializeStatic() {
        let search;
        if (window.opener && window.opener !== window) {
            search = new URL(window.location.href).search;
        }
        else {
            search = location.search;
        }
        search.substr(1).split("&").forEach(part => {
            let item = part.split("=");
            $("<x-property></x-property>")
                .attr("key", item[0])
                .attr("value", item[1])
                .appendTo(this._staticPropsTag);
        });
    }
    static(key, _default, default_type) {
        if (this._handle)
            return this._handle.static(key, _default, default_type);
        key = StaticSettings.keyify(key);
        return StaticSettings.resolveKey(key, _default, key => {
            let result = this._staticPropsTag.find("[key='" + key + "']");
            if (result.length > 0)
                return decodeURIComponent(result.last().attr('value'));
            return false;
        }, default_type);
    }
    deleteStatic(key) {
        if (this._handle) {
            this._handle.deleteStatic(key);
            return;
        }
        key = StaticSettings.keyify(key);
        let result = this._staticPropsTag.find("[key='" + key.key + "']");
        if (result.length != 0)
            result.detach();
    }
}
class Settings extends StaticSettings {
    constructor() {
        super();
        this.cacheGlobal = {};
        this.updated = false;
        const json = localStorage.getItem("settings.global");
        try {
            this.cacheGlobal = JSON.parse(json);
        }
        catch (error) {
            log.error(LogCategory.GENERAL, tr("Failed to load global settings!\nJson: %s\nError: %o"), json, error);
            const show_popup = () => {
                createErrorModal(tr("Failed to load global settings"), tr("Failed to load global client settings!\nLookup console for more information.")).open();
            };
            if (!loader.finished())
                loader.register_task(loader.Stage.LOADED, {
                    priority: 0,
                    name: "Settings error",
                    function: () => __awaiter(this, void 0, void 0, function* () { return show_popup(); })
                });
            else
                show_popup();
        }
        if (!this.cacheGlobal)
            this.cacheGlobal = {};
        this.saveWorker = setInterval(() => {
            if (this.updated)
                this.save();
        }, 5 * 1000);
    }
    static initialize() {
        settings = new Settings();
    }
    static_global(key, _default) {
        const actual_default = typeof (_default) === "undefined" && typeof (key) === "object" && 'default_value' in key ? key.default_value : _default;
        const default_object = { seed: Math.random() };
        let _static = this.static(key, default_object, typeof _default);
        if (_static !== default_object)
            return StaticSettings.transformStO(_static, actual_default);
        return this.global(key, actual_default);
    }
    global(key, _default) {
        const actual_default = typeof (_default) === "undefined" && typeof (key) === "object" && 'default_value' in key ? key.default_value : _default;
        return StaticSettings.resolveKey(Settings.keyify(key), actual_default, key => this.cacheGlobal[key]);
    }
    changeGlobal(key, value) {
        key = Settings.keyify(key);
        if (this.cacheGlobal[key.key] == value)
            return;
        this.updated = true;
        this.cacheGlobal[key.key] = StaticSettings.transformOtS(value);
        if (Settings.UPDATE_DIRECT)
            this.save();
    }
    save() {
        this.updated = false;
        let global = JSON.stringify(this.cacheGlobal);
        localStorage.setItem("settings.global", global);
        if (localStorage.save)
            localStorage.save();
    }
}
Settings.KEY_DISABLE_COSMETIC_SLOWDOWN = {
    key: 'disable_cosmetic_slowdown',
    description: 'Disable the cosmetic slowdows in some processes, like icon upload.'
};
Settings.KEY_DISABLE_CONTEXT_MENU = {
    key: 'disableContextMenu',
    description: 'Disable the context menu for the channel tree which allows to debug the DOM easier'
};
Settings.KEY_DISABLE_GLOBAL_CONTEXT_MENU = {
    key: 'disableGlobalContextMenu',
    description: 'Disable the general context menu prevention',
    default_value: false
};
Settings.KEY_DISABLE_UNLOAD_DIALOG = {
    key: 'disableUnloadDialog',
    description: 'Disables the unload popup on side closing'
};
Settings.KEY_DISABLE_VOICE = {
    key: 'disableVoice',
    description: 'Disables the voice bridge. If disabled, the audio and codec workers aren\'t required anymore'
};
Settings.KEY_DISABLE_MULTI_SESSION = {
    key: 'disableMultiSession',
    default_value: false,
    require_restart: true
};
Settings.KEY_LOAD_DUMMY_ERROR = {
    key: 'dummy_load_error',
    description: 'Triggers a loading error at the end of the loading process.'
};
/* Control bar */
Settings.KEY_CONTROL_MUTE_INPUT = {
    key: 'mute_input'
};
Settings.KEY_CONTROL_MUTE_OUTPUT = {
    key: 'mute_output'
};
Settings.KEY_CONTROL_SHOW_QUERIES = {
    key: 'show_server_queries'
};
Settings.KEY_CONTROL_CHANNEL_SUBSCRIBE_ALL = {
    key: 'channel_subscribe_all'
};
/* Connect parameters */
Settings.KEY_FLAG_CONNECT_DEFAULT = {
    key: 'connect_default'
};
Settings.KEY_CONNECT_ADDRESS = {
    key: 'connect_address'
};
Settings.KEY_CONNECT_PROFILE = {
    key: 'connect_profile'
};
Settings.KEY_CONNECT_USERNAME = {
    key: 'connect_username'
};
Settings.KEY_CONNECT_PASSWORD = {
    key: 'connect_password'
};
Settings.KEY_FLAG_CONNECT_PASSWORD = {
    key: 'connect_password_hashed'
};
Settings.KEY_CONNECT_HISTORY = {
    key: 'connect_history'
};
Settings.KEY_CONNECT_NO_DNSPROXY = {
    key: 'connect_no_dnsproxy',
    default_value: false
};
Settings.KEY_CERTIFICATE_CALLBACK = {
    key: 'certificate_callback'
};
/* sounds */
Settings.KEY_SOUND_MASTER = {
    key: 'audio_master_volume',
    default_value: 100
};
Settings.KEY_SOUND_MASTER_SOUNDS = {
    key: 'audio_master_volume_sounds',
    default_value: 100
};
Settings.KEY_CHAT_FIXED_TIMESTAMPS = {
    key: 'chat_fixed_timestamps',
    default_value: false,
    description: 'Enables fixed timestamps for chat messages and disabled the updating once (2 seconds ago... etc)'
};
Settings.KEY_CHAT_COLLOQUIAL_TIMESTAMPS = {
    key: 'chat_colloquial_timestamps',
    default_value: true,
    description: 'Enabled colloquial timestamp formatting like "Yesterday at ..." or "Today at ..."'
};
Settings.KEY_CHAT_COLORED_EMOJIES = {
    key: 'chat_colored_emojies',
    default_value: true,
    description: 'Enables colored emojies powered by Twemoji'
};
Settings.KEY_CHAT_TAG_URLS = {
    key: 'chat_tag_urls',
    default_value: true,
    description: 'Automatically link urls with [url]'
};
Settings.KEY_CHAT_ENABLE_MARKDOWN = {
    key: 'chat_enable_markdown',
    default_value: true,
    description: 'Enabled markdown chat support.'
};
Settings.KEY_CHAT_ENABLE_BBCODE = {
    key: 'chat_enable_bbcode',
    default_value: true,
    description: 'Enabled bbcode support in chat.'
};
Settings.KEY_SWITCH_INSTANT_CHAT = {
    key: 'switch_instant_chat',
    default_value: true,
    description: 'Directly switch to channel chat on channel select'
};
Settings.KEY_SWITCH_INSTANT_CLIENT = {
    key: 'switch_instant_client',
    default_value: true,
    description: 'Directly switch to client info on client select'
};
Settings.KEY_HOSTBANNER_BACKGROUND = {
    key: 'hostbanner_background',
    default_value: false,
    description: 'Enables a default background begind the hostbanner'
};
Settings.KEY_CHANNEL_EDIT_ADVANCED = {
    key: 'channel_edit_advanced',
    default_value: false,
    description: 'Edit channels in advanced mode with a lot more settings'
};
Settings.KEY_PERMISSIONS_SHOW_ALL = {
    key: 'permissions_show_all',
    default_value: false,
    description: 'Show all permissions even thou they dont make sense for the server/channel group'
};
Settings.KEY_TEAFORO_URL = {
    key: "teaforo_url",
    default_value: "https://forum.teaspeak.de/"
};
Settings.KEY_FONT_SIZE = {
    key: "font_size"
};
Settings.KEY_ICON_SIZE = {
    key: "icon_size",
    default_value: 100
};
Settings.KEY_LAST_INVITE_LINK_TYPE = {
    key: "last_invite_link_type",
    default_value: "tea-web"
};
Settings.FN_INVITE_LINK_SETTING = name => {
    return {
        key: 'invite_link_setting_' + name
    };
};
Settings.FN_SERVER_CHANNEL_SUBSCRIBE_MODE = channel => {
    return {
        key: 'channel_subscribe_mode_' + channel
    };
};
Settings.FN_PROFILE_RECORD = name => {
    return {
        key: 'profile_record' + name
    };
};
Settings.KEYS = (() => {
    const result = [];
    for (const key in Settings) {
        if (!key.toUpperCase().startsWith("KEY_"))
            continue;
        if (key.toUpperCase() == "KEYS")
            continue;
        result.push(key);
    }
    return result;
})();
class ServerSettings extends SettingsBase {
    constructor() {
        super();
        this.cacheServer = {};
        this._server_settings_updated = false;
        this._destroyed = false;
        this._server_save_worker = setInterval(() => {
            if (this._server_settings_updated)
                this.save();
        }, 5 * 1000);
    }
    destroy() {
        this._destroyed = true;
        this._server_unique_id = undefined;
        this.cacheServer = undefined;
        clearInterval(this._server_save_worker);
        this._server_save_worker = undefined;
    }
    server(key, _default) {
        if (this._destroyed)
            throw "destroyed";
        return StaticSettings.resolveKey(Settings.keyify(key), _default, key => this.cacheServer[key]);
    }
    changeServer(key, value) {
        if (this._destroyed)
            throw "destroyed";
        key = Settings.keyify(key);
        if (this.cacheServer[key.key] == value)
            return;
        this._server_settings_updated = true;
        this.cacheServer[key.key] = StaticSettings.transformOtS(value);
        if (Settings.UPDATE_DIRECT)
            this.save();
    }
    setServer(server_unique_id) {
        if (this._destroyed)
            throw "destroyed";
        if (this._server_unique_id) {
            this.save();
            this.cacheServer = {};
            this._server_unique_id = undefined;
        }
        this._server_unique_id = server_unique_id;
        if (this._server_unique_id) {
            const json = localStorage.getItem("settings.server_" + server_unique_id);
            try {
                this.cacheServer = JSON.parse(json);
            }
            catch (error) {
                log.error(LogCategory.GENERAL, tr("Failed to load server settings for server %s!\nJson: %s\nError: %o"), server_unique_id, json, error);
            }
            if (!this.cacheServer)
                this.cacheServer = {};
        }
    }
    save() {
        if (this._destroyed)
            throw "destroyed";
        this._server_settings_updated = false;
        if (this._server_unique_id) {
            let server = JSON.stringify(this.cacheServer);
            localStorage.setItem("settings.server_" + this._server_unique_id, server);
            if (localStorage.save)
                localStorage.save();
        }
    }
}
let settings;
//Used by CertAccept popup
if (!JSON.map_to) {
    JSON.map_to = function (object, json, variables, validator, variable_direction) {
        if (!validator)
            validator = (a, b) => true;
        if (!variables) {
            variables = [];
            if (!variable_direction || variable_direction == 0) {
                for (let field in json)
                    variables.push(field);
            }
            else if (variable_direction == 1) {
                for (let field in object)
                    variables.push(field);
            }
        }
        else if (!Array.isArray(variables)) {
            variables = [variables];
        }
        let updates = 0;
        for (let field of variables) {
            if (!json[field]) {
                console.trace(tr("Json does not contains %s"), field);
                continue;
            }
            if (!validator(field, json[field])) {
                console.trace(tr("Validator results in false for %s"), field);
                continue;
            }
            if (JSON.map_field_to(object, json[field], field))
                updates++;
        }
        return updates;
    };
}
if (!JSON.map_field_to) {
    JSON.map_field_to = function (object, value, field) {
        let field_type = typeof (object[field]);
        let new_object;
        if (field_type == "string" || field_type == "object" || field_type == "undefined")
            new_object = value;
        else if (field_type == "number")
            new_object = parseFloat(value);
        else if (field_type == "boolean")
            new_object = value == "1" || value == "true";
        else {
            console.warn(tr("Invalid object type %s for entry %s"), field_type, field);
            return false;
        }
        if (new_object === object[field])
            return false;
        object[field] = new_object;
        return true;
    };
}
if (!Array.prototype.remove) {
    Array.prototype.remove = function (elem) {
        const index = this.indexOf(elem, 0);
        if (index > -1) {
            this.splice(index, 1);
            return true;
        }
        return false;
    };
}
if (!Array.prototype.pop_front) {
    Array.prototype.pop_front = function () {
        if (this.length == 0)
            return undefined;
        return this.splice(0, 1)[0];
    };
}
if (!Array.prototype.last) {
    Array.prototype.last = function () {
        if (this.length == 0)
            return undefined;
        return this[this.length - 1];
    };
}
if (typeof ($) !== "undefined") {
    if (!$.spawn) {
        $.spawn = function (tagName) {
            return $(document.createElement(tagName));
        };
    }
    if (!$.fn.renderTag) {
        $.fn.renderTag = function (values) {
            let result;
            if (this.render) {
                result = $(this.render(values));
            }
            else {
                const template = window.jsrender.render[this.attr("id")];
                if (!template) {
                    console.error("Tried to render template %o, but template is not available!", this.attr("id"));
                    throw "missing template " + this.attr("id");
                }
                /*
                result = window.jsrender.templates("tmpl_permission_entry", $("#tmpl_permission_entry").html());
                result = window.jsrender.templates("xxx", this.html());
                */
                result = template(values);
                result = $(result);
            }
            result.find("node").each((index, element) => {
                $(element).replaceWith(values[$(element).attr("key")] || (values[0] || [])[$(element).attr("key")]);
            });
            return result;
        };
    }
    if (!$.fn.hasScrollBar)
        $.fn.hasScrollBar = function (direction) {
            if (this.length <= 0)
                return false;
            const scroll_height = this.get(0).scrollHeight > this.height();
            const scroll_width = this.get(0).scrollWidth > this.width();
            if (typeof (direction) === "string") {
                if (direction === "height")
                    return scroll_height;
                if (direction === "width")
                    return scroll_width;
            }
            return scroll_width || scroll_height;
        };
    if (!$.fn.visible_height)
        $.fn.visible_height = function () {
            const original_style = this.attr("style");
            this.css({
                position: 'absolute!important',
                visibility: 'hidden!important',
                display: 'block!important'
            });
            const result = this.height();
            this.attr("style", original_style || "");
            return result;
        };
    if (!$.fn.visible_width)
        $.fn.visible_width = function () {
            const original_style = this.attr("style");
            this.css({
                position: 'absolute!important',
                visibility: 'hidden!important',
                display: 'block!important'
            });
            const result = this.width();
            this.attr("style", original_style || "");
            return result;
        };
    if (!$.fn.firstParent)
        $.fn.firstParent = function (selector) {
            if (this.is(selector))
                return this;
            return this.parent(selector);
        };
}
if (!String.prototype.format) {
    String.prototype.format = function () {
        const args = arguments;
        let array = args.length == 1 && $.isArray(args[0]);
        return this.replace(/\{\{|\}\}|\{(\d+)\}/g, function (m, n) {
            if (m == "{{") {
                return "{";
            }
            if (m == "}}") {
                return "}";
            }
            return array ? args[0][n] : args[n];
        });
    };
}
function concatenate(resultConstructor, ...arrays) {
    let totalLength = 0;
    for (const arr of arrays) {
        totalLength += arr.length;
    }
    const result = new resultConstructor(totalLength);
    let offset = 0;
    for (const arr of arrays) {
        result.set(arr, offset);
        offset += arr.length;
    }
    return result;
}
function formatDate(secs) {
    let years = Math.floor(secs / (60 * 60 * 24 * 365));
    let days = Math.floor(secs / (60 * 60 * 24)) % 365;
    let hours = Math.floor(secs / (60 * 60)) % 24;
    let minutes = Math.floor(secs / 60) % 60;
    let seconds = Math.floor(secs % 60);
    let result = "";
    if (years > 0)
        result += years + " " + tr("years") + " ";
    if (years > 0 || days > 0)
        result += days + " " + tr("days") + " ";
    if (years > 0 || days > 0 || hours > 0)
        result += hours + " " + tr("hours") + " ";
    if (years > 0 || days > 0 || hours > 0 || minutes > 0)
        result += minutes + " " + tr("minutes") + " ";
    if (years > 0 || days > 0 || hours > 0 || minutes > 0 || seconds > 0)
        result += seconds + " " + tr("seconds") + " ";
    else
        result = tr("now") + " ";
    return result.substr(0, result.length - 1);
}
function calculate_width(text) {
    let element = $.spawn("div");
    element.text(text)
        .css("display", "none")
        .css("margin", 0);
    $("body").append(element);
    let size = element.width();
    element.detach();
    return size;
}
//Used by CertAccept popup
var LogCategory;
(function (LogCategory) {
    LogCategory[LogCategory["CHANNEL"] = 0] = "CHANNEL";
    LogCategory[LogCategory["CHANNEL_PROPERTIES"] = 1] = "CHANNEL_PROPERTIES";
    LogCategory[LogCategory["CLIENT"] = 2] = "CLIENT";
    LogCategory[LogCategory["BOOKMARKS"] = 3] = "BOOKMARKS";
    LogCategory[LogCategory["SERVER"] = 4] = "SERVER";
    LogCategory[LogCategory["PERMISSIONS"] = 5] = "PERMISSIONS";
    LogCategory[LogCategory["GENERAL"] = 6] = "GENERAL";
    LogCategory[LogCategory["NETWORKING"] = 7] = "NETWORKING";
    LogCategory[LogCategory["VOICE"] = 8] = "VOICE";
    LogCategory[LogCategory["CHAT"] = 9] = "CHAT";
    LogCategory[LogCategory["AUDIO"] = 10] = "AUDIO";
    LogCategory[LogCategory["I18N"] = 11] = "I18N";
    LogCategory[LogCategory["IPC"] = 12] = "IPC";
    LogCategory[LogCategory["IDENTITIES"] = 13] = "IDENTITIES";
    LogCategory[LogCategory["STATISTICS"] = 14] = "STATISTICS";
})(LogCategory || (LogCategory = {}));
var log;
(function (log_1) {
    let LogType;
    (function (LogType) {
        LogType[LogType["TRACE"] = 0] = "TRACE";
        LogType[LogType["DEBUG"] = 1] = "DEBUG";
        LogType[LogType["INFO"] = 2] = "INFO";
        LogType[LogType["WARNING"] = 3] = "WARNING";
        LogType[LogType["ERROR"] = 4] = "ERROR";
    })(LogType = log_1.LogType || (log_1.LogType = {}));
    let category_mapping = new Map([
        [LogCategory.CHANNEL, "Channel    "],
        [LogCategory.CHANNEL_PROPERTIES, "Channel    "],
        [LogCategory.CLIENT, "Client     "],
        [LogCategory.SERVER, "Server     "],
        [LogCategory.BOOKMARKS, "Bookmark   "],
        [LogCategory.PERMISSIONS, "Permission "],
        [LogCategory.GENERAL, "General    "],
        [LogCategory.NETWORKING, "Network    "],
        [LogCategory.VOICE, "Voice      "],
        [LogCategory.AUDIO, "Audio      "],
        [LogCategory.CHANNEL, "Chat       "],
        [LogCategory.I18N, "I18N       "],
        [LogCategory.IDENTITIES, "Identities "],
        [LogCategory.IPC, "IPC        "],
        [LogCategory.STATISTICS, "Statistics "]
    ]);
    log_1.enabled_mapping = new Map([
        [LogCategory.CHANNEL, true],
        [LogCategory.CHANNEL_PROPERTIES, false],
        [LogCategory.CLIENT, true],
        [LogCategory.SERVER, true],
        [LogCategory.BOOKMARKS, true],
        [LogCategory.PERMISSIONS, true],
        [LogCategory.GENERAL, true],
        [LogCategory.NETWORKING, true],
        [LogCategory.VOICE, true],
        [LogCategory.AUDIO, true],
        [LogCategory.CHAT, true],
        [LogCategory.I18N, false],
        [LogCategory.IDENTITIES, true],
        [LogCategory.IPC, true],
        [LogCategory.STATISTICS, true]
    ]);
    //Values will be overridden by initialize()
    log_1.level_mapping = new Map([
        [LogType.TRACE, true],
        [LogType.DEBUG, true],
        [LogType.INFO, true],
        [LogType.WARNING, true],
        [LogType.ERROR, true]
    ]);
    let GroupMode;
    (function (GroupMode) {
        GroupMode[GroupMode["NATIVE"] = 0] = "NATIVE";
        GroupMode[GroupMode["PREFIX"] = 1] = "PREFIX";
    })(GroupMode || (GroupMode = {}));
    const group_mode = GroupMode.PREFIX;
    //Category Example: <url>?log.i18n.enabled=0
    //Level Example A: <url>?log.level.trace.enabled=0
    //Level Example B: <url>?log.level=0
    function initialize(default_level) {
        for (const category of Object.keys(LogCategory).map(e => parseInt(e))) {
            if (isNaN(category))
                continue;
            const category_name = LogCategory[category].toLowerCase();
            log_1.enabled_mapping.set(category, settings.static_global("log." + category_name.toLowerCase() + ".enabled", log_1.enabled_mapping.get(category)));
        }
        const base_level = settings.static_global("log.level", default_level);
        for (const level of Object.keys(LogType).map(e => parseInt(e))) {
            if (isNaN(level))
                continue;
            const level_name = LogType[level].toLowerCase();
            log_1.level_mapping.set(level, settings.static_global("log." + level_name + ".enabled", level >= base_level));
        }
    }
    log_1.initialize = initialize;
    function logDirect(type, message, ...optionalParams) {
        if (!log_1.level_mapping.get(type))
            return;
        switch (type) {
            case LogType.TRACE:
            case LogType.DEBUG:
                console.debug(message, ...optionalParams);
                break;
            case LogType.INFO:
                console.log(message, ...optionalParams);
                break;
            case LogType.WARNING:
                console.warn(message, ...optionalParams);
                break;
            case LogType.ERROR:
                console.error(message, ...optionalParams);
                break;
        }
    }
    function log(type, category, message, ...optionalParams) {
        if (!log_1.enabled_mapping.get(category))
            return;
        optionalParams.unshift(category_mapping.get(category));
        message = "[%s] " + message;
        logDirect(type, message, ...optionalParams);
    }
    log_1.log = log;
    function trace(category, message, ...optionalParams) {
        log(LogType.TRACE, category, message, ...optionalParams);
    }
    log_1.trace = trace;
    function debug(category, message, ...optionalParams) {
        log(LogType.DEBUG, category, message, ...optionalParams);
    }
    log_1.debug = debug;
    function info(category, message, ...optionalParams) {
        log(LogType.INFO, category, message, ...optionalParams);
    }
    log_1.info = info;
    function warn(category, message, ...optionalParams) {
        log(LogType.WARNING, category, message, ...optionalParams);
    }
    log_1.warn = warn;
    function error(category, message, ...optionalParams) {
        log(LogType.ERROR, category, message, ...optionalParams);
    }
    log_1.error = error;
    function group(level, category, name, ...optionalParams) {
        name = "[%s] " + name;
        optionalParams.unshift(category_mapping.get(category));
        return new Group(group_mode, level, category, name, optionalParams);
    }
    log_1.group = group;
    function table(level, category, title, arguments) {
        if (group_mode == GroupMode.NATIVE) {
            console.groupCollapsed(title);
            console.table(arguments);
            console.groupEnd();
        }
        else {
            if (!log_1.enabled_mapping.get(category) || !log_1.level_mapping.get(level))
                return;
            logDirect(level, tr("Snipped table \"%s\""), title);
        }
    }
    log_1.table = table;
    class Group {
        constructor(mode, level, category, name, optionalParams, owner = undefined) {
            this.owner = undefined;
            this._collapsed = false;
            this.initialized = false;
            this.level = level;
            this.mode = mode;
            this.category = category;
            this.name = name;
            this.optionalParams = optionalParams;
            this.enabled = log_1.enabled_mapping.get(category);
        }
        group(level, name, ...optionalParams) {
            return new Group(this.mode, level, this.category, name, optionalParams, this);
        }
        collapsed(flag = true) {
            this._collapsed = flag;
            return this;
        }
        log(message, ...optionalParams) {
            if (!this.enabled)
                return this;
            if (!this.initialized) {
                if (this.mode == GroupMode.NATIVE) {
                    if (this._collapsed && console.groupCollapsed)
                        console.groupCollapsed(this.name, ...this.optionalParams);
                    else
                        console.group(this.name, ...this.optionalParams);
                }
                else {
                    this._log_prefix = "  ";
                    let parent = this.owner;
                    while (parent) {
                        if (parent.mode == GroupMode.PREFIX)
                            this._log_prefix = this._log_prefix + parent._log_prefix;
                        else
                            break;
                    }
                }
                this.initialized = true;
            }
            if (this.mode == GroupMode.NATIVE)
                logDirect(this.level, message, ...optionalParams);
            else {
                logDirect(this.level, "[%s] " + this._log_prefix + message, category_mapping.get(this.category), ...optionalParams);
            }
            return this;
        }
        end() {
            if (this.initialized) {
                if (this.mode == GroupMode.NATIVE)
                    console.groupEnd();
            }
        }
        get prefix() {
            return this._log_prefix;
        }
        set prefix(prefix) {
            this._log_prefix = prefix;
        }
    }
    log_1.Group = Group;
})(log || (log = {}));
var LogType = log.LogType;
