var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var loader;
(function (loader) {
    let config;
    (function (config) {
        config.loader_groups = false;
        config.verbose = false;
        config.error = true;
    })(config = loader.config || (loader.config = {}));
    let Stage;
    (function (Stage) {
        /*
            loading loader required files (incl this)
         */
        Stage[Stage["INITIALIZING"] = 0] = "INITIALIZING";
        /*
            setting up the loading process
         */
        Stage[Stage["SETUP"] = 1] = "SETUP";
        /*
            loading all style sheet files
         */
        Stage[Stage["STYLE"] = 2] = "STYLE";
        /*
            loading all javascript files
         */
        Stage[Stage["JAVASCRIPT"] = 3] = "JAVASCRIPT";
        /*
            loading all template files
         */
        Stage[Stage["TEMPLATES"] = 4] = "TEMPLATES";
        /*
            initializing static/global stuff
         */
        Stage[Stage["JAVASCRIPT_INITIALIZING"] = 5] = "JAVASCRIPT_INITIALIZING";
        /*
            finalizing load process
         */
        Stage[Stage["FINALIZING"] = 6] = "FINALIZING";
        /*
            invoking main task
         */
        Stage[Stage["LOADED"] = 7] = "LOADED";
        Stage[Stage["DONE"] = 8] = "DONE";
    })(Stage = loader.Stage || (loader.Stage = {}));
    let cache_tag;
    let current_stage = undefined;
    const tasks = {};
    /* test if all files shall be load from cache or fetch again */
    function loader_cache_tag() {
        const app_version = (() => {
            const version_node = document.getElementById("app_version");
            if (!version_node)
                return undefined;
            const version = version_node.hasAttribute("value") ? version_node.getAttribute("value") : undefined;
            if (!version)
                return undefined;
            if (!version || version == "unknown" || version.replace(/0+/, "").length == 0)
                return undefined;
            return version;
        })();
        if (config.verbose)
            console.log("Found current app version: %o", app_version);
        if (!app_version) {
            /* TODO add warning */
            cache_tag = "?_ts=" + Date.now();
            return;
        }
        const cached_version = localStorage.getItem("cached_version");
        if (!cached_version || cached_version != app_version) {
            loader.register_task(loader.Stage.LOADED, {
                priority: 0,
                name: "cached version updater",
                function: () => __awaiter(this, void 0, void 0, function* () {
                    localStorage.setItem("cached_version", app_version);
                })
            });
        }
        cache_tag = "?_version=" + app_version;
    }
    function get_cache_version() { return cache_tag; }
    loader.get_cache_version = get_cache_version;
    function finished() {
        return current_stage == Stage.DONE;
    }
    loader.finished = finished;
    function running() { return typeof (current_stage) !== "undefined"; }
    loader.running = running;
    function register_task(stage, task) {
        if (current_stage > stage) {
            if (config.error)
                console.warn("Register loading task, but it had already been finished. Executing task anyways!");
            task.function().catch(error => {
                if (config.error) {
                    console.error("Failed to execute delayed loader task!");
                    console.log(" - %s: %o", task.name, error);
                }
                loader.critical_error(error);
            });
            return;
        }
        const task_array = tasks[stage] || [];
        task_array.push(task);
        tasks[stage] = task_array.sort((a, b) => a.priority - b.priority);
    }
    loader.register_task = register_task;
    function execute() {
        return __awaiter(this, void 0, void 0, function* () {
            document.getElementById("loader-overlay").classList.add("started");
            loader_cache_tag();
            const load_begin = Date.now();
            let begin = 0;
            let end = Date.now();
            while (current_stage <= Stage.LOADED || typeof (current_stage) === "undefined") {
                let current_tasks = [];
                while ((tasks[current_stage] || []).length > 0) {
                    if (current_tasks.length == 0 || current_tasks[0].priority == tasks[current_stage][0].priority) {
                        current_tasks.push(tasks[current_stage].pop());
                    }
                    else
                        break;
                }
                const errors = [];
                const promises = [];
                for (const task of current_tasks) {
                    try {
                        if (config.verbose)
                            console.debug("Executing loader %s (%d)", task.name, task.priority);
                        promises.push(task.function().catch(error => {
                            errors.push({
                                task: task,
                                error: error
                            });
                            return Promise.resolve();
                        }));
                    }
                    catch (error) {
                        errors.push({
                            task: task,
                            error: error
                        });
                    }
                }
                if (promises.length > 0) {
                    yield Promise.all([...promises]);
                }
                if (errors.length > 0) {
                    if (config.loader_groups)
                        console.groupEnd();
                    console.error("Failed to execute loader. The following tasks failed (%d):", errors.length);
                    for (const error of errors)
                        console.error("  - %s: %o", error.task.name, error.error);
                    throw "failed to process step " + Stage[current_stage];
                }
                if (current_tasks.length == 0) {
                    if (typeof (current_stage) === "undefined") {
                        current_stage = -1;
                        if (config.verbose)
                            console.debug("[loader] Booting app");
                    }
                    else if (current_stage < Stage.INITIALIZING) {
                        if (config.loader_groups)
                            console.groupEnd();
                        if (config.verbose)
                            console.debug("[loader] Entering next state (%s). Last state took %dms", Stage[current_stage + 1], (end = Date.now()) - begin);
                    }
                    else {
                        if (config.loader_groups)
                            console.groupEnd();
                        if (config.verbose)
                            console.debug("[loader] Finish invoke took %dms", (end = Date.now()) - begin);
                    }
                    begin = end;
                    current_stage += 1;
                    if (current_stage != Stage.DONE && config.loader_groups)
                        console.groupCollapsed("Executing loading stage %s", Stage[current_stage]);
                }
            }
            /* cleanup */
            {
                _script_promises = {};
            }
            if (config.verbose)
                console.debug("[loader] finished loader. (Total time: %dms)", Date.now() - load_begin);
        });
    }
    loader.execute = execute;
    function execute_managed() {
        loader.execute().then(() => {
            if (config.verbose) {
                let message;
                if (typeof (window.tr) !== "undefined")
                    message = tr("App loaded successfully!");
                else
                    message = "App loaded successfully!";
                if (typeof (log) !== "undefined") {
                    /* We're having our log module */
                    log.info(LogCategory.GENERAL, message);
                }
                else {
                    console.log(message);
                }
            }
        }).catch(error => {
            console.error("App loading failed: %o", error);
            loader.critical_error("Failed to execute loader", "Lookup the console for more detail");
        });
    }
    loader.execute_managed = execute_managed;
    function script_name(path) {
        if (Array.isArray(path)) {
            let buffer = "";
            let _or = " or ";
            for (let entry of path)
                buffer += _or + script_name(entry);
            return buffer.slice(_or.length);
        }
        else if (typeof (path) === "string")
            return "<code>" + path + "</code>";
        else
            return "<code>" + path.url + "</code>";
    }
    class SyntaxError {
        constructor(source) {
            this.source = source;
        }
    }
    let _script_promises = {};
    function load_script(path) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(path)) { //We have some fallback
                return load_script(path[0]).catch(error => {
                    if (error instanceof SyntaxError)
                        return Promise.reject(error.source);
                    if (path.length > 1)
                        return load_script(path.slice(1));
                    return Promise.reject(error);
                });
            }
            else {
                const source = typeof (path) === "string" ? { url: path, depends: [] } : path;
                if (source.url.length == 0)
                    return Promise.resolve();
                return _script_promises[source.url] = (() => __awaiter(this, void 0, void 0, function* () {
                    /* await depends */
                    for (const depend of source.depends) {
                        if (!_script_promises[depend])
                            throw "Missing dependency " + depend;
                        yield _script_promises[depend];
                    }
                    const tag = document.createElement("script");
                    yield new Promise((resolve, reject) => {
                        let error = false;
                        const error_handler = (event) => {
                            if (event.filename == tag.src && event.message.indexOf("Illegal constructor") == -1) { //Our tag throw an uncaught error
                                if (config.verbose)
                                    console.log("msg: %o, url: %o, line: %o, col: %o, error: %o", event.message, event.filename, event.lineno, event.colno, event.error);
                                window.removeEventListener('error', error_handler);
                                reject(new SyntaxError(event.error));
                                event.preventDefault();
                                error = true;
                            }
                        };
                        window.addEventListener('error', error_handler);
                        const cleanup = () => {
                            tag.onerror = undefined;
                            tag.onload = undefined;
                            clearTimeout(timeout_handle);
                            window.removeEventListener('error', error_handler);
                        };
                        const timeout_handle = setTimeout(() => {
                            cleanup();
                            reject("timeout");
                        }, 5000);
                        tag.type = "application/javascript";
                        tag.async = true;
                        tag.defer = true;
                        tag.onerror = error => {
                            cleanup();
                            tag.remove();
                            reject(error);
                        };
                        tag.onload = () => {
                            cleanup();
                            if (config.verbose)
                                console.debug("Script %o loaded", path);
                            setTimeout(resolve, 100);
                        };
                        document.getElementById("scripts").appendChild(tag);
                        tag.src = source.url + (cache_tag || "");
                    });
                }))();
            }
        });
    }
    loader.load_script = load_script;
    function load_scripts(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = [];
            const errors = [];
            for (const script of paths)
                promises.push(load_script(script).catch(error => {
                    errors.push({
                        script: script,
                        error: error
                    });
                    return Promise.resolve();
                }));
            yield Promise.all([...promises]);
            if (errors.length > 0) {
                if (config.error) {
                    console.error("Failed to load the following scripts:");
                    for (const script of errors)
                        console.log(" - %o: %o", script.script, script.error);
                }
                loader.critical_error("Failed to load script " + script_name(errors[0].script) + " <br>" + "View the browser console for more information!");
                throw "failed to load script " + script_name(errors[0].script);
            }
        });
    }
    loader.load_scripts = load_scripts;
    function load_style(path) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(path)) { //We have some fallback
                return load_style(path[0]).catch(error => {
                    if (error instanceof SyntaxError)
                        return Promise.reject(error.source);
                    if (path.length > 1)
                        return load_script(path.slice(1));
                    return Promise.reject(error);
                });
            }
            else {
                if (!path) {
                    return Promise.resolve();
                }
                return new Promise((resolve, reject) => {
                    const tag = document.createElement("link");
                    let error = false;
                    const error_handler = (event) => {
                        if (config.verbose)
                            console.log("msg: %o, url: %o, line: %o, col: %o, error: %o", event.message, event.filename, event.lineno, event.colno, event.error);
                        if (event.filename == tag.href) { //FIXME!
                            window.removeEventListener('error', error_handler);
                            reject(new SyntaxError(event.error));
                            event.preventDefault();
                            error = true;
                        }
                    };
                    window.addEventListener('error', error_handler);
                    tag.type = "text/css";
                    tag.rel = "stylesheet";
                    const cleanup = () => {
                        tag.onerror = undefined;
                        tag.onload = undefined;
                        clearTimeout(timeout_handle);
                        window.removeEventListener('error', error_handler);
                    };
                    const timeout_handle = setTimeout(() => {
                        cleanup();
                        reject("timeout");
                    }, 5000);
                    tag.onerror = error => {
                        cleanup();
                        tag.remove();
                        if (config.error)
                            console.error("File load error for file %s: %o", path, error);
                        reject("failed to load file " + path);
                    };
                    tag.onload = () => {
                        cleanup();
                        {
                            const css = tag.sheet;
                            const rules = css.cssRules;
                            const rules_remove = [];
                            const rules_add = [];
                            for (let index = 0; index < rules.length; index++) {
                                const rule = rules.item(index);
                                let rule_text = rule.cssText;
                                if (rule.cssText.indexOf("%%base_path%%") != -1) {
                                    rules_remove.push(index);
                                    rules_add.push(rule_text.replace("%%base_path%%", document.location.origin + document.location.pathname));
                                }
                            }
                            for (const index of rules_remove.sort((a, b) => b > a ? 1 : 0)) {
                                if (css.removeRule)
                                    css.removeRule(index);
                                else
                                    css.deleteRule(index);
                            }
                            for (const rule of rules_add)
                                css.insertRule(rule, rules_remove[0]);
                        }
                        if (config.verbose)
                            console.debug("Style sheet %o loaded", path);
                        setTimeout(resolve, 100);
                    };
                    document.getElementById("style").appendChild(tag);
                    tag.href = path + (cache_tag || "");
                });
            }
        });
    }
    loader.load_style = load_style;
    function load_styles(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = [];
            const errors = [];
            for (const sheet of paths)
                promises.push(load_style(sheet).catch(error => {
                    errors.push({
                        sheet: sheet,
                        error: error
                    });
                    return Promise.resolve();
                }));
            yield Promise.all([...promises]);
            if (errors.length > 0) {
                if (loader.config.error) {
                    console.error("Failed to load the following style sheet:");
                    for (const sheet of errors)
                        console.log(" - %o: %o", sheet.sheet, sheet.error);
                }
                loader.critical_error("Failed to load style sheet " + script_name(errors[0].sheet) + " <br>" + "View the browser console for more information!");
                throw "failed to load style sheet " + script_name(errors[0].sheet);
            }
        });
    }
    loader.load_styles = load_styles;
    let _callback_critical_error;
    let _callback_critical_called = false;
    function critical_error(message, detail) {
        if (_callback_critical_called) {
            console.warn("[CRITICAL] %s", message);
            if (typeof (detail) === "string")
                console.warn("[CRITICAL] %s", detail);
            return;
        }
        _callback_critical_called = true;
        if (_callback_critical_error) {
            _callback_critical_error(message, detail);
            return;
        }
        /* default handling */
        let tag = document.getElementById("critical-load");
        {
            const error_tags = tag.getElementsByClassName("error");
            error_tags[0].innerHTML = message;
        }
        if (typeof (detail) === "string") {
            let node_detail = tag.getElementsByClassName("detail")[0];
            node_detail.innerHTML = detail;
        }
        tag.classList.add("shown");
    }
    loader.critical_error = critical_error;
    function critical_error_handler(handler, override) {
        if ((typeof (handler) === "object" && handler !== _callback_critical_error) || override)
            _callback_critical_error = handler;
        return _callback_critical_error;
    }
    loader.critical_error_handler = critical_error_handler;
})(loader || (loader = {}));
{
    const hello_world = () => {
        const clog = console.log;
        const print_security = () => {
            {
                const css = [
                    "display: block",
                    "text-align: center",
                    "font-size: 42px",
                    "font-weight: bold",
                    "-webkit-text-stroke: 2px black",
                    "color: red"
                ].join(";");
                clog("%c ", "font-size: 100px;");
                clog("%cSecurity warning:", css);
            }
            {
                const css = [
                    "display: block",
                    "text-align: center",
                    "font-size: 18px",
                    "font-weight: bold"
                ].join(";");
                clog("%cPasting anything in here could give attackers access to your data.", css);
                clog("%cUnless you understand exactly what you are doing, close this window and stay safe.", css);
                clog("%c ", "font-size: 100px;");
            }
        };
        /* print the hello world */
        {
            const css = [
                "display: block",
                "text-align: center",
                "font-size: 72px",
                "font-weight: bold",
                "-webkit-text-stroke: 2px black",
                "color: #18BC9C"
            ].join(";");
            clog("%cHey, hold on!", css);
        }
        {
            const css = [
                "display: block",
                "text-align: center",
                "font-size: 26px",
                "font-weight: bold"
            ].join(";");
            const css_2 = [
                "display: block",
                "text-align: center",
                "font-size: 26px",
                "font-weight: bold",
                "color: blue"
            ].join(";");
            const display_detect = /./;
            display_detect.toString = function () { print_security(); return ""; };
            clog("%cLovely to see you using and debugging the TeaSpeak Web client.", css);
            clog("%cIf you have some good ideas or already done some incredible changes,", css);
            clog("%cyou'll be may interested to share them here: %chttps://github.com/TeaSpeak/TeaWeb", css, css_2);
            clog("%c ", display_detect);
        }
    };
    try { /* lets try to print it as VM code :)*/
        let hello_world_code = hello_world.toString();
        hello_world_code = hello_world_code.substr(hello_world_code.indexOf('() => {') + 8);
        hello_world_code = hello_world_code.substring(0, hello_world_code.lastIndexOf("}"));
        //Look aheads are not possible with firefox
        //hello_world_code = hello_world_code.replace(/(?<!const|let)(?<=^([^"'/]|"[^"]*"|'[^']*'|`[^`]*`|\/[^/]*\/)*) /gm, ""); /* replace all spaces */
        hello_world_code = hello_world_code.replace(/[\n\r]/g, ""); /* replace as new lines */
        eval(hello_world_code);
    }
    catch (e) {
        console.error(e);
        hello_world();
    }
}
//FUN: loader_ignore_age=0&loader_default_duration=1500&loader_default_age=5000
let _fadeout_warned = false;
function fadeoutLoader(duration = undefined, minAge = undefined, ignoreAge = undefined) {
    if (typeof ($) === "undefined") {
        if (!_fadeout_warned)
            console.warn("Could not fadeout loader screen. Missing jquery functions.");
        _fadeout_warned = true;
        return;
    }
    let settingsDefined = typeof (StaticSettings) !== "undefined";
    if (!duration) {
        if (settingsDefined)
            duration = StaticSettings.instance.static("loader_default_duration", 750);
        else
            duration = 750;
    }
    if (!minAge) {
        if (settingsDefined)
            minAge = StaticSettings.instance.static("loader_default_age", 1750);
        else
            minAge = 750;
    }
    if (!ignoreAge) {
        if (settingsDefined)
            ignoreAge = StaticSettings.instance.static("loader_ignore_age", false);
        else
            ignoreAge = false;
    }
    /*
    let age = Date.now() - app.appLoaded;
    if(age < minAge && !ignoreAge) {
        setTimeout(() => fadeoutLoader(duration, 0, true), minAge - age);
        return;
    }
    */
    $(".loader .bookshelf_wrapper").animate({ top: 0, opacity: 0 }, duration);
    $(".loader .half").animate({ width: 0 }, duration, () => {
        $(".loader").detach();
    });
}
/* set a timeout here, so if this script is merged with the actual loader (like in rel mode) the actual could load this manually here */
setTimeout(() => {
    if (loader.running()) {
        if (loader.config.verbose)
            console.debug("Do not execute debug loading");
        return;
    }
    loader.register_task(loader.Stage.INITIALIZING, {
        priority: 100,
        function: () => __awaiter(this, void 0, void 0, function* () {
            let loader_type;
            /*
            location.search.replace(/(?:^\?|&)([a-zA-Z_]+)=([.a-zA-Z0-9]+)(?=$|&)/g, (_, key: string, value: string) => {
                if(key.toLowerCase() == "loader_target")
                    loader_type = value;
                return "";
            });
             */
            for (const node of document.head.getElementsByTagName("meta")) {
                if (node.name === "app-loader-target") {
                    loader_type = node.content;
                    if (loader_type)
                        break;
                }
            }
            loader_type = loader_type || "app";
            if (loader_type === "app") {
                try {
                    yield loader.load_scripts(["loader/app.js"]);
                }
                catch (error) {
                    console.error("Failed to load main app script: %o", error);
                    loader.critical_error("Failed to load main app script", error);
                    throw "app loader failed";
                }
            }
            else if (loader_type === "certaccept") {
                try {
                    yield loader.load_scripts(["loader/certaccept.js"]);
                }
                catch (error) {
                    console.error("Failed to load cert accept script: %o", error);
                    loader.critical_error("Failed to load cert accept script", error);
                    throw "app loader failed";
                }
            }
            else {
                loader.critical_error("Missing loader target: " + loader_type);
                throw "Missing loader target: " + loader_type;
            }
        }),
        name: "loading target"
    });
    loader.execute_managed();
}, 0);
/// <reference path="loader.ts" />
var app;
(function (app) {
    let Type;
    (function (Type) {
        Type[Type["UNKNOWN"] = 0] = "UNKNOWN";
        Type[Type["CLIENT_RELEASE"] = 1] = "CLIENT_RELEASE";
        Type[Type["CLIENT_DEBUG"] = 2] = "CLIENT_DEBUG";
        Type[Type["WEB_DEBUG"] = 3] = "WEB_DEBUG";
        Type[Type["WEB_RELEASE"] = 4] = "WEB_RELEASE";
    })(Type = app.Type || (app.Type = {}));
    app.type = Type.UNKNOWN;
    function is_web() {
        return app.type == Type.WEB_RELEASE || app.type == Type.WEB_DEBUG;
    }
    app.is_web = is_web;
    let _ui_version;
    function ui_version() {
        if (typeof (_ui_version) !== "string") {
            const version_node = document.getElementById("app_version");
            if (!version_node)
                return undefined;
            const version = version_node.hasAttribute("value") ? version_node.getAttribute("value") : undefined;
            if (!version)
                return undefined;
            return (_ui_version = version);
        }
        return _ui_version;
    }
    app.ui_version = ui_version;
})(app || (app = {}));
/* all javascript loaders */
const loader_javascript = {
    detect_type: () => __awaiter(this, void 0, void 0, function* () {
        if (window.require) {
            const request = new Request("js/proto.js");
            let file_path = request.url;
            if (!file_path.startsWith("file://"))
                throw "Invalid file path (" + file_path + ")";
            file_path = file_path.substring(process.platform === "win32" ? 8 : 7);
            const fs = require('fs');
            if (fs.existsSync(file_path)) {
                app.type = app.Type.CLIENT_DEBUG;
            }
            else {
                app.type = app.Type.CLIENT_RELEASE;
            }
        }
        else {
            /* test if js/proto.js is available. If so we're in debug mode */
            const request = new XMLHttpRequest();
            request.open('GET', "js/proto.js?_ts=" + Date.now(), true);
            yield new Promise((resolve, reject) => {
                request.onreadystatechange = () => {
                    if (request.readyState === 4) {
                        if (request.status === 404) {
                            app.type = app.Type.WEB_RELEASE;
                        }
                        else {
                            app.type = app.Type.WEB_DEBUG;
                        }
                        resolve();
                    }
                };
                request.onerror = () => {
                    reject("Failed to detect app type");
                };
                request.send();
            });
        }
    }),
    load_scripts: () => __awaiter(this, void 0, void 0, function* () {
        if (!window.require) {
            yield loader.load_script(["vendor/jquery/jquery.min.js"]);
        }
        else {
            /*
            loader.register_task(loader.Stage.JAVASCRIPT_INITIALIZING, {
                name: "forum sync",
                priority: 10,
                function: async () => {
                    forum.sync_main();
                }
            });
            */
        }
        yield loader.load_script(["vendor/DOMPurify/purify.min.js"]);
        yield loader.load_script("vendor/jsrender/jsrender.min.js");
        yield loader.load_scripts([
            ["vendor/xbbcode/src/parser.js"],
            ["vendor/moment/moment.js"],
            ["vendor/twemoji/twemoji.min.js", ""],
            ["vendor/highlight/highlight.pack.js", ""],
            ["vendor/remarkable/remarkable.min.js", ""],
            ["adapter/adapter-latest.js", "https://webrtc.github.io/adapter/adapter-latest.js"]
        ]);
        yield loader.load_scripts([
            ["vendor/emoji-picker/src/jquery.lsxemojipicker.js"]
        ]);
        if (app.type == app.Type.WEB_RELEASE || app.type == app.Type.CLIENT_RELEASE) {
            loader.register_task(loader.Stage.JAVASCRIPT, {
                name: "scripts release",
                priority: 20,
                function: loader_javascript.load_release
            });
        }
        else {
            loader.register_task(loader.Stage.JAVASCRIPT, {
                name: "scripts debug",
                priority: 20,
                function: loader_javascript.load_scripts_debug
            });
        }
    }),
    load_scripts_debug: () => __awaiter(this, void 0, void 0, function* () {
        /* test if we're loading as TeaClient or WebClient */
        if (!window.require) {
            loader.register_task(loader.Stage.JAVASCRIPT, {
                name: "javascript web",
                priority: 10,
                function: loader_javascript.load_scripts_debug_web
            });
        }
        else {
            loader.register_task(loader.Stage.JAVASCRIPT, {
                name: "javascript client",
                priority: 10,
                function: loader_javascript.load_scripts_debug_client
            });
        }
        /* load some extends classes */
        yield loader.load_scripts([
            ["js/connection/ConnectionBase.js"]
        ]);
        /* load the main app */
        yield loader.load_scripts([
            //Load general API's
            "js/proto.js",
            "js/i18n/localize.js",
            "js/i18n/country.js",
            "js/log.js",
            "js/sound/Sounds.js",
            "js/utils/helpers.js",
            "js/crypto/sha.js",
            "js/crypto/hex.js",
            "js/crypto/asn1.js",
            "js/crypto/crc32.js",
            //load the profiles
            "js/profiles/ConnectionProfile.js",
            "js/profiles/Identity.js",
            "js/profiles/identities/teaspeak-forum.js",
            //Basic UI elements
            "js/ui/elements/context_divider.js",
            "js/ui/elements/context_menu.js",
            "js/ui/elements/modal.js",
            "js/ui/elements/tab.js",
            "js/ui/elements/slider.js",
            "js/ui/elements/tooltip.js",
            "js/ui/elements/net_graph.js",
            //Load permissions
            "js/permission/PermissionManager.js",
            "js/permission/GroupManager.js",
            //Load UI
            "js/ui/modal/ModalAbout.js",
            "js/ui/modal/ModalAvatar.js",
            "js/ui/modal/ModalAvatarList.js",
            "js/ui/modal/ModalClientInfo.js",
            "js/ui/modal/ModalChannelInfo.js",
            "js/ui/modal/ModalServerInfo.js",
            "js/ui/modal/ModalServerInfoBandwidth.js",
            "js/ui/modal/ModalQuery.js",
            "js/ui/modal/ModalQueryManage.js",
            "js/ui/modal/ModalPlaylistList.js",
            "js/ui/modal/ModalPlaylistEdit.js",
            "js/ui/modal/ModalBookmarks.js",
            "js/ui/modal/ModalConnect.js",
            "js/ui/modal/ModalSettings.js",
            "js/ui/modal/ModalCreateChannel.js",
            "js/ui/modal/ModalServerEdit.js",
            "js/ui/modal/ModalChangeVolume.js",
            "js/ui/modal/ModalChangeLatency.js",
            "js/ui/modal/ModalBanClient.js",
            "js/ui/modal/ModalIconSelect.js",
            "js/ui/modal/ModalInvite.js",
            "js/ui/modal/ModalIdentity.js",
            "js/ui/modal/ModalBanList.js",
            "js/ui/modal/ModalYesNo.js",
            "js/ui/modal/ModalPoke.js",
            "js/ui/modal/ModalKeySelect.js",
            "js/ui/modal/ModalGroupAssignment.js",
            "js/ui/modal/permission/ModalPermissionEdit.js",
            { url: "js/ui/modal/permission/SenselessPermissions.js", depends: ["js/permission/PermissionManager.js"] },
            { url: "js/ui/modal/permission/CanvasPermissionEditor.js", depends: ["js/ui/modal/permission/ModalPermissionEdit.js"] },
            { url: "js/ui/modal/permission/HTMLPermissionEditor.js", depends: ["js/ui/modal/permission/ModalPermissionEdit.js"] },
            "js/ui/channel.js",
            "js/ui/client.js",
            "js/ui/server.js",
            "js/ui/view.js",
            "js/ui/client_move.js",
            "js/ui/htmltags.js",
            "js/ui/frames/ControlBar.js",
            "js/ui/frames/chat.js",
            "js/ui/frames/chat_frame.js",
            "js/ui/frames/connection_handlers.js",
            "js/ui/frames/server_log.js",
            "js/ui/frames/hostbanner.js",
            "js/ui/frames/MenuBar.js",
            //Load audio
            "js/voice/RecorderBase.js",
            "js/voice/RecorderProfile.js",
            //Load general stuff
            "js/settings.js",
            "js/bookmarks.js",
            "js/FileManager.js",
            "js/ConnectionHandler.js",
            "js/BrowserIPC.js",
            "js/dns.js",
            //Connection
            "js/connection/CommandHandler.js",
            "js/connection/CommandHelper.js",
            "js/connection/HandshakeHandler.js",
            "js/connection/ServerConnectionDeclaration.js",
            "js/stats.js",
            "js/PPTListener.js",
            "js/profiles/identities/NameIdentity.js",
            "js/profiles/identities/TeaForumIdentity.js",
            "js/profiles/identities/TeamSpeakIdentity.js",
        ]);
        yield loader.load_script("js/main.js");
    }),
    load_scripts_debug_web: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_scripts([
            ["js/audio/AudioPlayer.js"],
            ["js/audio/WebCodec.js"],
            ["js/WebPPTListener.js"],
            "js/voice/AudioResampler.js",
            "js/voice/JavascriptRecorder.js",
            "js/voice/VoiceHandler.js",
            "js/voice/VoiceClient.js",
            //Connection
            "js/connection/ServerConnection.js",
            //Load codec
            "js/codec/Codec.js",
            "js/codec/BasicCodec.js",
            { url: "js/codec/CodecWrapperWorker.js", depends: ["js/codec/BasicCodec.js"] },
        ]);
    }),
    load_scripts_debug_client: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_scripts([]);
    }),
    load_release: () => __awaiter(this, void 0, void 0, function* () {
        console.log("Load for release!");
        yield loader.load_scripts([
            //Load general API's
            ["js/client.min.js", "js/client.js"]
        ]);
    })
};
const loader_webassembly = {
    test_webassembly: () => __awaiter(this, void 0, void 0, function* () {
        /* We dont required WebAssembly anymore for fundamental functions, only for auto decoding
        if(typeof (WebAssembly) === "undefined" || typeof (WebAssembly.compile) === "undefined") {
            console.log(navigator.browserSpecs);
            if (navigator.browserSpecs.name == 'Safari') {
                if (parseInt(navigator.browserSpecs.version) < 11) {
                    displayCriticalError("You require Safari 11 or higher to use the web client!<br>Safari " + navigator.browserSpecs.version + " does not support WebAssambly!");
                    return;
                }
            }
            else {
                // Do something for all other browsers.
            }
            displayCriticalError("You require WebAssembly for TeaSpeak-Web!");
            throw "Missing web assembly";
        }
        */
    })
};
const loader_style = {
    load_style: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_styles([
            "vendor/xbbcode/src/xbbcode.css"
        ]);
        yield loader.load_styles([
            "vendor/emoji-picker/src/jquery.lsxemojipicker.css"
        ]);
        yield loader.load_styles([
            ["vendor/highlight/styles/darcula.css", ""],
        ]);
        if (app.type == app.Type.WEB_DEBUG || app.type == app.Type.CLIENT_DEBUG) {
            yield loader_style.load_style_debug();
        }
        else {
            yield loader_style.load_style_release();
        }
    }),
    load_style_debug: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_styles([
            "css/static/main.css",
            "css/static/main-layout.css",
            "css/static/helptag.css",
            "css/static/scroll.css",
            "css/static/channel-tree.css",
            "css/static/ts/tab.css",
            "css/static/ts/chat.css",
            "css/static/ts/icons.css",
            "css/static/ts/icons_em.css",
            "css/static/ts/country.css",
            "css/static/general.css",
            "css/static/modal.css",
            "css/static/modals.css",
            "css/static/modal-about.css",
            "css/static/modal-avatar.css",
            "css/static/modal-icons.css",
            "css/static/modal-bookmarks.css",
            "css/static/modal-connect.css",
            "css/static/modal-channel.css",
            "css/static/modal-query.css",
            "css/static/modal-volume.css",
            "css/static/modal-latency.css",
            "css/static/modal-invite.css",
            "css/static/modal-playlist.css",
            "css/static/modal-banlist.css",
            "css/static/modal-banclient.css",
            "css/static/modal-channelinfo.css",
            "css/static/modal-clientinfo.css",
            "css/static/modal-serverinfo.css",
            "css/static/modal-serverinfobandwidth.css",
            "css/static/modal-identity.css",
            "css/static/modal-settings.css",
            "css/static/modal-poke.css",
            "css/static/modal-server.css",
            "css/static/modal-keyselect.css",
            "css/static/modal-permissions.css",
            "css/static/modal-group-assignment.css",
            "css/static/music/info_plate.css",
            "css/static/frame/SelectInfo.css",
            "css/static/control_bar.css",
            "css/static/context_menu.css",
            "css/static/frame-chat.css",
            "css/static/connection_handlers.css",
            "css/static/server-log.css",
            "css/static/htmltags.css",
            "css/static/hostbanner.css",
            "css/static/menu-bar.css"
        ]);
    }),
    load_style_release: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_styles([
            "css/static/base.css",
            "css/static/main.css",
        ]);
    })
};
function load_templates() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield $.ajax("templates.html" + loader.get_cache_version());
            let node = document.createElement("html");
            node.innerHTML = response;
            let tags;
            if (node.getElementsByTagName("body").length > 0)
                tags = node.getElementsByTagName("body")[0].children;
            else
                tags = node.children;
            let root = document.getElementById("templates");
            if (!root) {
                loader.critical_error("Failed to find template tag!");
                return;
            }
            while (tags.length > 0) {
                let tag = tags.item(0);
                root.appendChild(tag);
            }
        }
        catch (error) {
            loader.critical_error("Failed to find template tag!");
            throw "template error";
        }
    });
}
/* register tasks */
loader.register_task(loader.Stage.INITIALIZING, {
    name: "safari fix",
    function: () => __awaiter(this, void 0, void 0, function* () {
        /* safari remove "fix" */
        if (Element.prototype.remove === undefined)
            Object.defineProperty(Element.prototype, "remove", {
                enumerable: false,
                configurable: false,
                writable: false,
                value: function () {
                    this.parentElement.removeChild(this);
                }
            });
    }),
    priority: 50
});
loader.register_task(loader.Stage.INITIALIZING, {
    name: "Browser detection",
    function: () => __awaiter(this, void 0, void 0, function* () {
        navigator.browserSpecs = (function () {
            let ua = navigator.userAgent, tem, M = ua.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i) || [];
            if (/trident/i.test(M[1])) {
                tem = /\brv[ :]+(\d+)/g.exec(ua) || [];
                return { name: 'IE', version: (tem[1] || '') };
            }
            if (M[1] === 'Chrome') {
                tem = ua.match(/\b(OPR|Edge)\/(\d+)/);
                if (tem != null)
                    return { name: tem[1].replace('OPR', 'Opera'), version: tem[2] };
            }
            M = M[2] ? [M[1], M[2]] : [navigator.appName, navigator.appVersion, '-?'];
            if ((tem = ua.match(/version\/(\d+)/i)) != null)
                M.splice(1, 1, tem[1]);
            return { name: M[0], version: M[1] };
        })();
        console.log("Resolved browser manufacturer to \"%s\" version \"%s\"", navigator.browserSpecs.name, navigator.browserSpecs.version);
    }),
    priority: 30
});
loader.register_task(loader.Stage.INITIALIZING, {
    name: "secure tester",
    function: () => __awaiter(this, void 0, void 0, function* () {
        /* we need https or localhost to use some things like the storage API */
        if (typeof isSecureContext === "undefined")
            window["isSecureContext"] = location.protocol !== 'https:' && location.hostname !== 'localhost';
        if (!isSecureContext) {
            loader.critical_error("TeaWeb cant run on unsecured sides.", "App requires to be loaded via HTTPS!");
            throw "App requires a secure context!";
        }
    }),
    priority: 20
});
loader.register_task(loader.Stage.INITIALIZING, {
    name: "webassembly tester",
    function: loader_webassembly.test_webassembly,
    priority: 20
});
loader.register_task(loader.Stage.INITIALIZING, {
    name: "app type test",
    function: loader_javascript.detect_type,
    priority: 20
});
loader.register_task(loader.Stage.JAVASCRIPT, {
    name: "javascript",
    function: loader_javascript.load_scripts,
    priority: 10
});
loader.register_task(loader.Stage.STYLE, {
    name: "style",
    function: loader_style.load_style,
    priority: 10
});
loader.register_task(loader.Stage.TEMPLATES, {
    name: "templates",
    function: load_templates,
    priority: 10
});
loader.register_task(loader.Stage.LOADED, {
    name: "loaded handler",
    function: () => __awaiter(this, void 0, void 0, function* () {
        fadeoutLoader();
    }),
    priority: 5
});
loader.register_task(loader.Stage.LOADED, {
    name: "error task",
    function: () => __awaiter(this, void 0, void 0, function* () {
        if (Settings.instance.static(Settings.KEY_LOAD_DUMMY_ERROR, false)) {
            loader.critical_error("The tea is cold!", "Argh, this is evil! Cold tea dosn't taste good.");
            throw "The tea is cold!";
        }
    }),
    priority: 20
});
loader.register_task(loader.Stage.JAVASCRIPT_INITIALIZING, {
    name: "lsx emoji picker setup",
    function: () => __awaiter(this, void 0, void 0, function* () { return yield window.setup_lsx_emoji_picker({ twemoji: typeof (window.twemoji) !== "undefined" }); }),
    priority: 10
});
loader.register_task(loader.Stage.SETUP, {
    name: "page setup",
    function: () => __awaiter(this, void 0, void 0, function* () {
        const body = document.body;
        /* top menu */
        {
            const container = document.createElement("div");
            container.setAttribute('id', "top-menu-bar");
            body.append(container);
        }
        /* template containers */
        {
            const container = document.createElement("div");
            container.setAttribute('id', "templates");
            body.append(container);
        }
        /* sounds container */
        {
            const container = document.createElement("div");
            container.setAttribute('id', "sounds");
            body.append(container);
        }
        /* mouse move container */
        {
            const container = document.createElement("div");
            container.setAttribute('id', "mouse-move");
            const inner_container = document.createElement("div");
            inner_container.classList.add("container");
            container.append(inner_container);
            body.append(container);
        }
        /* tooltip container */
        {
            const container = document.createElement("div");
            container.setAttribute('id', "global-tooltip");
            container.append(document.createElement("a"));
            body.append(container);
        }
    }),
    priority: 10
});
/* test if we're getting loaded within a TeaClient preview window */
loader.register_task(loader.Stage.SETUP, {
    name: "TeaClient tester",
    function: () => __awaiter(this, void 0, void 0, function* () {
        //@ts-ignore
        if (typeof __teaclient_preview_notice !== "undefined" && typeof __teaclient_preview_error !== "undefined") {
            loader.critical_error("Why you're opening TeaWeb within the TeaSpeak client?!");
            throw "we're already a TeaClient!";
        }
    }),
    priority: 100
});
loader.register_task(loader.Stage.JAVASCRIPT_INITIALIZING, {
    name: "log enabled initialisation",
    function: () => __awaiter(this, void 0, void 0, function* () { return log.initialize(app.type === app.Type.CLIENT_DEBUG || app.type === app.Type.WEB_DEBUG ? log.LogType.TRACE : log.LogType.INFO); }),
    priority: 150
});
window["Module"] = (window["Module"] || {});
/* TeaClient */
if (window.require) {
    const path = require("path");
    const remote = require('electron').remote;
    module.paths.push(path.join(remote.app.getAppPath(), "/modules"));
    module.paths.push(path.join(path.dirname(remote.getGlobal("browser-root")), "js"));
    const connector = require("renderer");
    console.log(connector);
    loader.register_task(loader.Stage.INITIALIZING, {
        name: "teaclient initialize",
        function: connector.initialize,
        priority: 40
    });
}
if (!loader.running()) {
    /* we know that we want to load the app */
    loader.execute_managed();
}
//# sourceMappingURL=loader_app.js.map