var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var loader;
(function (loader) {
    let config;
    (function (config) {
        config.loader_groups = false;
        config.verbose = false;
        config.error = true;
    })(config = loader.config || (loader.config = {}));
    let Stage;
    (function (Stage) {
        /*
            loading loader required files (incl this)
         */
        Stage[Stage["INITIALIZING"] = 0] = "INITIALIZING";
        /*
            setting up the loading process
         */
        Stage[Stage["SETUP"] = 1] = "SETUP";
        /*
            loading all style sheet files
         */
        Stage[Stage["STYLE"] = 2] = "STYLE";
        /*
            loading all javascript files
         */
        Stage[Stage["JAVASCRIPT"] = 3] = "JAVASCRIPT";
        /*
            loading all template files
         */
        Stage[Stage["TEMPLATES"] = 4] = "TEMPLATES";
        /*
            initializing static/global stuff
         */
        Stage[Stage["JAVASCRIPT_INITIALIZING"] = 5] = "JAVASCRIPT_INITIALIZING";
        /*
            finalizing load process
         */
        Stage[Stage["FINALIZING"] = 6] = "FINALIZING";
        /*
            invoking main task
         */
        Stage[Stage["LOADED"] = 7] = "LOADED";
        Stage[Stage["DONE"] = 8] = "DONE";
    })(Stage = loader.Stage || (loader.Stage = {}));
    let cache_tag;
    let current_stage = undefined;
    const tasks = {};
    /* test if all files shall be load from cache or fetch again */
    function loader_cache_tag() {
        const app_version = (() => {
            const version_node = document.getElementById("app_version");
            if (!version_node)
                return undefined;
            const version = version_node.hasAttribute("value") ? version_node.getAttribute("value") : undefined;
            if (!version)
                return undefined;
            if (!version || version == "unknown" || version.replace(/0+/, "").length == 0)
                return undefined;
            return version;
        })();
        if (config.verbose)
            console.log("Found current app version: %o", app_version);
        if (!app_version) {
            /* TODO add warning */
            cache_tag = "?_ts=" + Date.now();
            return;
        }
        const cached_version = localStorage.getItem("cached_version");
        if (!cached_version || cached_version != app_version) {
            loader.register_task(loader.Stage.LOADED, {
                priority: 0,
                name: "cached version updater",
                function: () => __awaiter(this, void 0, void 0, function* () {
                    localStorage.setItem("cached_version", app_version);
                })
            });
        }
        cache_tag = "?_version=" + app_version;
    }
    function get_cache_version() { return cache_tag; }
    loader.get_cache_version = get_cache_version;
    function finished() {
        return current_stage == Stage.DONE;
    }
    loader.finished = finished;
    function running() { return typeof (current_stage) !== "undefined"; }
    loader.running = running;
    function register_task(stage, task) {
        if (current_stage > stage) {
            if (config.error)
                console.warn("Register loading task, but it had already been finished. Executing task anyways!");
            task.function().catch(error => {
                if (config.error) {
                    console.error("Failed to execute delayed loader task!");
                    console.log(" - %s: %o", task.name, error);
                }
                loader.critical_error(error);
            });
            return;
        }
        const task_array = tasks[stage] || [];
        task_array.push(task);
        tasks[stage] = task_array.sort((a, b) => a.priority - b.priority);
    }
    loader.register_task = register_task;
    function execute() {
        return __awaiter(this, void 0, void 0, function* () {
            document.getElementById("loader-overlay").classList.add("started");
            loader_cache_tag();
            const load_begin = Date.now();
            let begin = 0;
            let end = Date.now();
            while (current_stage <= Stage.LOADED || typeof (current_stage) === "undefined") {
                let current_tasks = [];
                while ((tasks[current_stage] || []).length > 0) {
                    if (current_tasks.length == 0 || current_tasks[0].priority == tasks[current_stage][0].priority) {
                        current_tasks.push(tasks[current_stage].pop());
                    }
                    else
                        break;
                }
                const errors = [];
                const promises = [];
                for (const task of current_tasks) {
                    try {
                        if (config.verbose)
                            console.debug("Executing loader %s (%d)", task.name, task.priority);
                        promises.push(task.function().catch(error => {
                            errors.push({
                                task: task,
                                error: error
                            });
                            return Promise.resolve();
                        }));
                    }
                    catch (error) {
                        errors.push({
                            task: task,
                            error: error
                        });
                    }
                }
                if (promises.length > 0) {
                    yield Promise.all([...promises]);
                }
                if (errors.length > 0) {
                    if (config.loader_groups)
                        console.groupEnd();
                    console.error("Failed to execute loader. The following tasks failed (%d):", errors.length);
                    for (const error of errors)
                        console.error("  - %s: %o", error.task.name, error.error);
                    throw "failed to process step " + Stage[current_stage];
                }
                if (current_tasks.length == 0) {
                    if (typeof (current_stage) === "undefined") {
                        current_stage = -1;
                        if (config.verbose)
                            console.debug("[loader] Booting app");
                    }
                    else if (current_stage < Stage.INITIALIZING) {
                        if (config.loader_groups)
                            console.groupEnd();
                        if (config.verbose)
                            console.debug("[loader] Entering next state (%s). Last state took %dms", Stage[current_stage + 1], (end = Date.now()) - begin);
                    }
                    else {
                        if (config.loader_groups)
                            console.groupEnd();
                        if (config.verbose)
                            console.debug("[loader] Finish invoke took %dms", (end = Date.now()) - begin);
                    }
                    begin = end;
                    current_stage += 1;
                    if (current_stage != Stage.DONE && config.loader_groups)
                        console.groupCollapsed("Executing loading stage %s", Stage[current_stage]);
                }
            }
            /* cleanup */
            {
                _script_promises = {};
            }
            if (config.verbose)
                console.debug("[loader] finished loader. (Total time: %dms)", Date.now() - load_begin);
        });
    }
    loader.execute = execute;
    function execute_managed() {
        loader.execute().then(() => {
            if (config.verbose) {
                let message;
                if (typeof (window.tr) !== "undefined")
                    message = tr("App loaded successfully!");
                else
                    message = "App loaded successfully!";
                if (typeof (log) !== "undefined") {
                    /* We're having our log module */
                    log.info(LogCategory.GENERAL, message);
                }
                else {
                    console.log(message);
                }
            }
        }).catch(error => {
            console.error("App loading failed: %o", error);
            loader.critical_error("Failed to execute loader", "Lookup the console for more detail");
        });
    }
    loader.execute_managed = execute_managed;
    function script_name(path) {
        if (Array.isArray(path)) {
            let buffer = "";
            let _or = " or ";
            for (let entry of path)
                buffer += _or + script_name(entry);
            return buffer.slice(_or.length);
        }
        else if (typeof (path) === "string")
            return "<code>" + path + "</code>";
        else
            return "<code>" + path.url + "</code>";
    }
    class SyntaxError {
        constructor(source) {
            this.source = source;
        }
    }
    let _script_promises = {};
    function load_script(path) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(path)) { //We have some fallback
                return load_script(path[0]).catch(error => {
                    if (error instanceof SyntaxError)
                        return Promise.reject(error.source);
                    if (path.length > 1)
                        return load_script(path.slice(1));
                    return Promise.reject(error);
                });
            }
            else {
                const source = typeof (path) === "string" ? { url: path, depends: [] } : path;
                if (source.url.length == 0)
                    return Promise.resolve();
                return _script_promises[source.url] = (() => __awaiter(this, void 0, void 0, function* () {
                    /* await depends */
                    for (const depend of source.depends) {
                        if (!_script_promises[depend])
                            throw "Missing dependency " + depend;
                        yield _script_promises[depend];
                    }
                    const tag = document.createElement("script");
                    yield new Promise((resolve, reject) => {
                        let error = false;
                        const error_handler = (event) => {
                            if (event.filename == tag.src && event.message.indexOf("Illegal constructor") == -1) { //Our tag throw an uncaught error
                                if (config.verbose)
                                    console.log("msg: %o, url: %o, line: %o, col: %o, error: %o", event.message, event.filename, event.lineno, event.colno, event.error);
                                window.removeEventListener('error', error_handler);
                                reject(new SyntaxError(event.error));
                                event.preventDefault();
                                error = true;
                            }
                        };
                        window.addEventListener('error', error_handler);
                        const cleanup = () => {
                            tag.onerror = undefined;
                            tag.onload = undefined;
                            clearTimeout(timeout_handle);
                            window.removeEventListener('error', error_handler);
                        };
                        const timeout_handle = setTimeout(() => {
                            cleanup();
                            reject("timeout");
                        }, 5000);
                        tag.type = "application/javascript";
                        tag.async = true;
                        tag.defer = true;
                        tag.onerror = error => {
                            cleanup();
                            tag.remove();
                            reject(error);
                        };
                        tag.onload = () => {
                            cleanup();
                            if (config.verbose)
                                console.debug("Script %o loaded", path);
                            setTimeout(resolve, 100);
                        };
                        document.getElementById("scripts").appendChild(tag);
                        tag.src = source.url + (cache_tag || "");
                    });
                }))();
            }
        });
    }
    loader.load_script = load_script;
    function load_scripts(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = [];
            const errors = [];
            for (const script of paths)
                promises.push(load_script(script).catch(error => {
                    errors.push({
                        script: script,
                        error: error
                    });
                    return Promise.resolve();
                }));
            yield Promise.all([...promises]);
            if (errors.length > 0) {
                if (config.error) {
                    console.error("Failed to load the following scripts:");
                    for (const script of errors)
                        console.log(" - %o: %o", script.script, script.error);
                }
                loader.critical_error("Failed to load script " + script_name(errors[0].script) + " <br>" + "View the browser console for more information!");
                throw "failed to load script " + script_name(errors[0].script);
            }
        });
    }
    loader.load_scripts = load_scripts;
    function load_style(path) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Array.isArray(path)) { //We have some fallback
                return load_style(path[0]).catch(error => {
                    if (error instanceof SyntaxError)
                        return Promise.reject(error.source);
                    if (path.length > 1)
                        return load_script(path.slice(1));
                    return Promise.reject(error);
                });
            }
            else {
                if (!path) {
                    return Promise.resolve();
                }
                return new Promise((resolve, reject) => {
                    const tag = document.createElement("link");
                    let error = false;
                    const error_handler = (event) => {
                        if (config.verbose)
                            console.log("msg: %o, url: %o, line: %o, col: %o, error: %o", event.message, event.filename, event.lineno, event.colno, event.error);
                        if (event.filename == tag.href) { //FIXME!
                            window.removeEventListener('error', error_handler);
                            reject(new SyntaxError(event.error));
                            event.preventDefault();
                            error = true;
                        }
                    };
                    window.addEventListener('error', error_handler);
                    tag.type = "text/css";
                    tag.rel = "stylesheet";
                    const cleanup = () => {
                        tag.onerror = undefined;
                        tag.onload = undefined;
                        clearTimeout(timeout_handle);
                        window.removeEventListener('error', error_handler);
                    };
                    const timeout_handle = setTimeout(() => {
                        cleanup();
                        reject("timeout");
                    }, 5000);
                    tag.onerror = error => {
                        cleanup();
                        tag.remove();
                        if (config.error)
                            console.error("File load error for file %s: %o", path, error);
                        reject("failed to load file " + path);
                    };
                    tag.onload = () => {
                        cleanup();
                        {
                            const css = tag.sheet;
                            const rules = css.cssRules;
                            const rules_remove = [];
                            const rules_add = [];
                            for (let index = 0; index < rules.length; index++) {
                                const rule = rules.item(index);
                                let rule_text = rule.cssText;
                                if (rule.cssText.indexOf("%%base_path%%") != -1) {
                                    rules_remove.push(index);
                                    rules_add.push(rule_text.replace("%%base_path%%", document.location.origin + document.location.pathname));
                                }
                            }
                            for (const index of rules_remove.sort((a, b) => b > a ? 1 : 0)) {
                                if (css.removeRule)
                                    css.removeRule(index);
                                else
                                    css.deleteRule(index);
                            }
                            for (const rule of rules_add)
                                css.insertRule(rule, rules_remove[0]);
                        }
                        if (config.verbose)
                            console.debug("Style sheet %o loaded", path);
                        setTimeout(resolve, 100);
                    };
                    document.getElementById("style").appendChild(tag);
                    tag.href = path + (cache_tag || "");
                });
            }
        });
    }
    loader.load_style = load_style;
    function load_styles(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = [];
            const errors = [];
            for (const sheet of paths)
                promises.push(load_style(sheet).catch(error => {
                    errors.push({
                        sheet: sheet,
                        error: error
                    });
                    return Promise.resolve();
                }));
            yield Promise.all([...promises]);
            if (errors.length > 0) {
                if (loader.config.error) {
                    console.error("Failed to load the following style sheet:");
                    for (const sheet of errors)
                        console.log(" - %o: %o", sheet.sheet, sheet.error);
                }
                loader.critical_error("Failed to load style sheet " + script_name(errors[0].sheet) + " <br>" + "View the browser console for more information!");
                throw "failed to load style sheet " + script_name(errors[0].sheet);
            }
        });
    }
    loader.load_styles = load_styles;
    let _callback_critical_error;
    let _callback_critical_called = false;
    function critical_error(message, detail) {
        if (_callback_critical_called) {
            console.warn("[CRITICAL] %s", message);
            if (typeof (detail) === "string")
                console.warn("[CRITICAL] %s", detail);
            return;
        }
        _callback_critical_called = true;
        if (_callback_critical_error) {
            _callback_critical_error(message, detail);
            return;
        }
        /* default handling */
        let tag = document.getElementById("critical-load");
        {
            const error_tags = tag.getElementsByClassName("error");
            error_tags[0].innerHTML = message;
        }
        if (typeof (detail) === "string") {
            let node_detail = tag.getElementsByClassName("detail")[0];
            node_detail.innerHTML = detail;
        }
        tag.classList.add("shown");
    }
    loader.critical_error = critical_error;
    function critical_error_handler(handler, override) {
        if ((typeof (handler) === "object" && handler !== _callback_critical_error) || override)
            _callback_critical_error = handler;
        return _callback_critical_error;
    }
    loader.critical_error_handler = critical_error_handler;
})(loader || (loader = {}));
{
    const hello_world = () => {
        const clog = console.log;
        const print_security = () => {
            {
                const css = [
                    "display: block",
                    "text-align: center",
                    "font-size: 42px",
                    "font-weight: bold",
                    "-webkit-text-stroke: 2px black",
                    "color: red"
                ].join(";");
                clog("%c ", "font-size: 100px;");
                clog("%cSecurity warning:", css);
            }
            {
                const css = [
                    "display: block",
                    "text-align: center",
                    "font-size: 18px",
                    "font-weight: bold"
                ].join(";");
                clog("%cPasting anything in here could give attackers access to your data.", css);
                clog("%cUnless you understand exactly what you are doing, close this window and stay safe.", css);
                clog("%c ", "font-size: 100px;");
            }
        };
        /* print the hello world */
        {
            const css = [
                "display: block",
                "text-align: center",
                "font-size: 72px",
                "font-weight: bold",
                "-webkit-text-stroke: 2px black",
                "color: #18BC9C"
            ].join(";");
            clog("%cHey, hold on!", css);
        }
        {
            const css = [
                "display: block",
                "text-align: center",
                "font-size: 26px",
                "font-weight: bold"
            ].join(";");
            const css_2 = [
                "display: block",
                "text-align: center",
                "font-size: 26px",
                "font-weight: bold",
                "color: blue"
            ].join(";");
            const display_detect = /./;
            display_detect.toString = function () { print_security(); return ""; };
            clog("%cLovely to see you using and debugging the TeaSpeak Web client.", css);
            clog("%cIf you have some good ideas or already done some incredible changes,", css);
            clog("%cyou'll be may interested to share them here: %chttps://github.com/TeaSpeak/TeaWeb", css, css_2);
            clog("%c ", display_detect);
        }
    };
    try { /* lets try to print it as VM code :)*/
        let hello_world_code = hello_world.toString();
        hello_world_code = hello_world_code.substr(hello_world_code.indexOf('() => {') + 8);
        hello_world_code = hello_world_code.substring(0, hello_world_code.lastIndexOf("}"));
        //Look aheads are not possible with firefox
        //hello_world_code = hello_world_code.replace(/(?<!const|let)(?<=^([^"'/]|"[^"]*"|'[^']*'|`[^`]*`|\/[^/]*\/)*) /gm, ""); /* replace all spaces */
        hello_world_code = hello_world_code.replace(/[\n\r]/g, ""); /* replace as new lines */
        eval(hello_world_code);
    }
    catch (e) {
        console.error(e);
        hello_world();
    }
}
//FUN: loader_ignore_age=0&loader_default_duration=1500&loader_default_age=5000
let _fadeout_warned = false;
function fadeoutLoader(duration = undefined, minAge = undefined, ignoreAge = undefined) {
    if (typeof ($) === "undefined") {
        if (!_fadeout_warned)
            console.warn("Could not fadeout loader screen. Missing jquery functions.");
        _fadeout_warned = true;
        return;
    }
    let settingsDefined = typeof (StaticSettings) !== "undefined";
    if (!duration) {
        if (settingsDefined)
            duration = StaticSettings.instance.static("loader_default_duration", 750);
        else
            duration = 750;
    }
    if (!minAge) {
        if (settingsDefined)
            minAge = StaticSettings.instance.static("loader_default_age", 1750);
        else
            minAge = 750;
    }
    if (!ignoreAge) {
        if (settingsDefined)
            ignoreAge = StaticSettings.instance.static("loader_ignore_age", false);
        else
            ignoreAge = false;
    }
    /*
    let age = Date.now() - app.appLoaded;
    if(age < minAge && !ignoreAge) {
        setTimeout(() => fadeoutLoader(duration, 0, true), minAge - age);
        return;
    }
    */
    $(".loader .bookshelf_wrapper").animate({ top: 0, opacity: 0 }, duration);
    $(".loader .half").animate({ width: 0 }, duration, () => {
        $(".loader").detach();
    });
}
/* set a timeout here, so if this script is merged with the actual loader (like in rel mode) the actual could load this manually here */
setTimeout(() => {
    if (loader.running()) {
        if (loader.config.verbose)
            console.debug("Do not execute debug loading");
        return;
    }
    loader.register_task(loader.Stage.INITIALIZING, {
        priority: 100,
        function: () => __awaiter(this, void 0, void 0, function* () {
            let loader_type;
            /*
            location.search.replace(/(?:^\?|&)([a-zA-Z_]+)=([.a-zA-Z0-9]+)(?=$|&)/g, (_, key: string, value: string) => {
                if(key.toLowerCase() == "loader_target")
                    loader_type = value;
                return "";
            });
             */
            for (const node of document.head.getElementsByTagName("meta")) {
                if (node.name === "app-loader-target") {
                    loader_type = node.content;
                    if (loader_type)
                        break;
                }
            }
            loader_type = loader_type || "app";
            if (loader_type === "app") {
                try {
                    yield loader.load_scripts(["loader/app.js"]);
                }
                catch (error) {
                    console.error("Failed to load main app script: %o", error);
                    loader.critical_error("Failed to load main app script", error);
                    throw "app loader failed";
                }
            }
            else if (loader_type === "certaccept") {
                try {
                    yield loader.load_scripts(["loader/certaccept.js"]);
                }
                catch (error) {
                    console.error("Failed to load cert accept script: %o", error);
                    loader.critical_error("Failed to load cert accept script", error);
                    throw "app loader failed";
                }
            }
            else {
                loader.critical_error("Missing loader target: " + loader_type);
                throw "Missing loader target: " + loader_type;
            }
        }),
        name: "loading target"
    });
    loader.execute_managed();
}, 0);
/// <reference path="loader.ts" />
let is_debug = false;
/* all javascript loaders */
const loader_javascript = {
    detect_type: () => __awaiter(this, void 0, void 0, function* () {
        /* test if js/proto.js is available. If so we're in debug mode */
        const request = new XMLHttpRequest();
        request.open('GET', 'js/proto.js', true);
        yield new Promise((resolve, reject) => {
            request.onreadystatechange = () => {
                if (request.readyState === 4) {
                    is_debug = request.status !== 404;
                    resolve();
                }
            };
            request.onerror = () => {
                reject("Failed to detect app type");
            };
            request.send();
        });
    }),
    load_scripts: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_script(["vendor/jquery/jquery.min.js"]);
        if (!is_debug) {
            loader.register_task(loader.Stage.JAVASCRIPT, {
                name: "scripts release",
                priority: 20,
                function: loader_javascript.load_release
            });
        }
        else {
            loader.register_task(loader.Stage.JAVASCRIPT, {
                name: "scripts debug",
                priority: 20,
                function: loader_javascript.load_scripts_debug
            });
        }
    }),
    load_scripts_debug: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_scripts([
            ["js/proto.js"],
            ["js/log.js"],
            ["js/BrowserIPC.js"],
            ["js/settings.js"],
            ["js/main.js"]
        ]);
    }),
    load_release: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_scripts([
            ["js/certaccept.min.js", "js/certaccept.js"]
        ]);
    })
};
const loader_style = {
    load_style: () => __awaiter(this, void 0, void 0, function* () {
        if (is_debug) {
            yield loader_style.load_style_debug();
        }
        else {
            yield loader_style.load_style_release();
        }
    }),
    load_style_debug: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_styles([
            "css/static/main.css",
        ]);
    }),
    load_style_release: () => __awaiter(this, void 0, void 0, function* () {
        yield loader.load_styles([
            "css/static/main.css",
        ]);
    })
};
loader.register_task(loader.Stage.INITIALIZING, {
    name: "app type test",
    function: loader_javascript.detect_type,
    priority: 20
});
loader.register_task(loader.Stage.JAVASCRIPT, {
    name: "javascript",
    function: loader_javascript.load_scripts,
    priority: 10
});
loader.register_task(loader.Stage.STYLE, {
    name: "style",
    function: loader_style.load_style,
    priority: 10
});
loader.register_task(loader.Stage.LOADED, {
    name: "loaded handler",
    function: () => __awaiter(this, void 0, void 0, function* () {
        fadeoutLoader();
    }),
    priority: 0
});
/* register tasks */
loader.register_task(loader.Stage.INITIALIZING, {
    name: "safari fix",
    function: () => __awaiter(this, void 0, void 0, function* () {
        /* safari remove "fix" */
        if (Element.prototype.remove === undefined)
            Object.defineProperty(Element.prototype, "remove", {
                enumerable: false,
                configurable: false,
                writable: false,
                value: function () {
                    this.parentElement.removeChild(this);
                }
            });
    }),
    priority: 50
});
loader.register_task(loader.Stage.JAVASCRIPT_INITIALIZING, {
    name: "settings initialisation",
    function: () => __awaiter(this, void 0, void 0, function* () { return Settings.initialize(); }),
    priority: 200
});
loader.register_task(loader.Stage.JAVASCRIPT_INITIALIZING, {
    name: "bipc initialisation",
    function: () => __awaiter(this, void 0, void 0, function* () { return bipc.setup(); }),
    priority: 100
});
loader.register_task(loader.Stage.JAVASCRIPT_INITIALIZING, {
    name: "log enabled initialisation",
    function: () => __awaiter(this, void 0, void 0, function* () { return log.initialize(is_debug ? log.LogType.TRACE : log.LogType.INFO); }),
    priority: 150
});
if (!loader.running()) {
    /* we know that we want to load the app */
    loader.execute_managed();
}
//# sourceMappingURL=loader_certaccept.js.map